#!/usr/bin/perl
#
# $Id: nessus_run.pl,v 1.19 2006/09/20 01:10:36 airwofl Exp $
#
################################################################################
#                Inprotect                #
#-----------------------------------------------------------------------------#
# Copyright (C) 2002 - 2006 http://inprotect.sourceforge.net/           #
#                                #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of version 2 of the GNU General Public License as published #
# by the Free Software Foundation.                    #
#                                #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                              #
#                                #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc.,     #
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA              #
#                                #
# Contact Information:                          #
# Inprotect Team (inprotect-devel@lists@sourceforge.net)          #
# http://inprotect.sourceforge.net/                   #
###############################################################################
# See the README.txt and/or help files for more information on how to use &   #
# configuration.                          #
# See the LICENSE.txt file for more information on the License this software  #
# is distributed under.                         #
#                                #
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.                 #
###############################################################################

use strict;
use warnings;
use POSIX qw(setsid);
use POSIX ":sys_wait_h";
use IO::Socket;
use Socket;
use DBI;

#Read settings from the inprotect.cfg file
use vars qw/%CONFIG/;
load_configs();

#Redirect output to the log file
my $nessuslog = "/var/log/inprotect/nessus_daemon.log";
my $debug = 0;
my $serverid = "$CONFIG{'SERVERID'}";
#my $serverid = "";

open(LOG,">>$nessuslog") or die "Failed to create $nessuslog: $!\n";
LOG->autoflush(1); # make LOG hot
*STDERR=*LOG;
*STDOUT=*LOG;

print localtime(time)." nessus_daemon: Start\n";

#Declare constants
use constant TRUE => 1;
use constant FALSE => 0;
use constant DEBUG => 1;

#Declare variables to use
my $time_to_die;
$time_to_die = 0;

# Only user root can run this program 
if ( $> != 0 ) {
    print localtime(time)." nessus_daemon: You must be a superuser to start this application\n";
    exit 1;
}

# Check if the program is currently running
if ( CheckRunning("nessus_daemon.pl") ) {
    print localtime(time)." nessus_daemon: Another instance of the daemon is currently running\n";
    exit 2;
}

# Become a daemon
print localtime(time)." nessus_daemon: Become a daemon\n";
daemonize();


# Trap fatal signals
$SIG{INT} = $SIG{QUIT} = $SIG{TERM} = $SIG{HUP} = \&signal_handler;
$SIG{PIPE} = "IGNORE";
$SIG{CHLD} = "IGNORE"; # prevent zombie processes


#Connecting to the database
my $dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}","$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
        PrintError => 0,
        RaiseError => 1,
        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");

# Go into an infinite loop and wake up every 5 secs
until ( $time_to_die ) {

   if(! $dbh->ping) {
      $dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}","$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
         PrintError => 0,
         RaiseError => 1,
         AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
   }

   # get the list of all scanners
   #Select server attributes
   my $sth = $dbh->prepare(qq{
      SELECT id, hostname
      FROM nessus_servers
      } );

   $sth->execute;
   my $nessus_servers = $sth->fetchall_hashref('id');

   $sth->finish;

   # prepare some SQL statements for execution inside the loop
   #Check for scans to stop
   my $sth_stssel = $dbh->prepare(qq{
      SELECT distinct process_id
      FROM nessus_scan
      WHERE status = "P"
      AND server_id=?
      });

   # loop through the scanner array
   foreach $serverid (keys(%{$nessus_servers})) {
      #if(DEBUG) {  print "    checking scanner $serverid\n"; }

      $sth_stssel->execute($serverid);

      my $stopid;
      $sth_stssel->bind_columns(\$stopid);
      while($sth_stssel->fetch) {
         if(DEBUG) { print localtime(time)." nessus_daemon: asked to kill $stopid\n"; }
         # since nessus_scan.pl spawns children to do the actual nessus client
         # invocation, we can get the nessus client PID by looking for any
         # process with a PPID=stopid
         my $ppid = `ps -o pid= --ppid $stopid`;
         chomp($ppid);
         while($ppid ne "") {
            if(DEBUG) { print localtime(time)." nessus_daemon: killing $ppid\n"; }
            kill 7, $ppid;
            $ppid = `ps -o pid= --ppid $stopid`;
            chomp($ppid);
         } 
      } #end of loop

      sleep 1;

   } # end of foreach loop for nessus_servers
   $sth_stssel->finish;

} # end of infinite loop

#Close DB connection
$dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");

print localtime(time)." nessus_daemon: Exiting\n";
close LOG;

exit 0;


# Become a daemon 
sub daemonize {
    my ( $pid );
    chdir "/" or die "nessus_run: Can't chdir to /: $!\n";
    defined($pid = fork) or die "nessus_run: Can't fork: $!\n";
    exit if $pid;
    setsid or die "nessus_run: Can't start a new session: $!\n";
    umask 0;
}

# Signal handler for 1,2,3 and 15 
sub signal_handler {
    $time_to_die = 1;
}
## Checks entries in the process table for the program 
## Returns true if entry is found 
sub CheckRunning {
    my ($pgname) = @_;
    my ($exists, $pid);
    $pid = $$;
    open(PROC,"ps -eo '%p %a'|") or die "nessus_daemon: Failed to fork : $!\n";

    $exists = 0;
    while(<PROC>) {
       next if /grep|vi|vim|tail|$pid/;
       if ( /$pgname/ ) {
          $exists = 1;
          last;
       }
    }
    close PROC;
    return $exists;
}

sub load_configs
{
   my $configfile="/etc/inprotect/inprotect.cfg";
   my $noconfig=0;
   open(CONF,"<$configfile") || $noconfig++;
   my @CONFILE=<CONF>;
   close(CONF);

   if ($noconfig) { print localtime(time)." nessus_run: No inprotect.cfg file found.\n"; }
   foreach my $line (@CONFILE)
   {
      $line =~ s/\#.*$//;
      chomp($line);
      $line =~ s/\s+$//;
      $line =~ s/^\s+//;
      if ($line eq "") { next; }
      my @temp=split(/=/,$line,2);
      if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
   }
   return;
}
