#!/usr/bin/perl
#
# $Id: nessus_scan.pl,v 1.19 2006/09/20 01:10:36 airwofl Exp $
#
################################################################################
#                Inprotect                #
#-----------------------------------------------------------------------------#
# Copyright (C) 2002 - 2006 http://inprotect.sourceforge.net/           #
#                                #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of version 2 of the GNU General Public License as published #
# by the Free Software Foundation.                    #
#                                #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                              #
#                                #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc.,     #
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA              #
#                                #
# Contact Information:                          #
# Inprotect Team (inprotect-devel@lists@sourceforge.net)          #
# http://inprotect.sourceforge.net/                   #
###############################################################################
# See the README.txt and/or help files for more information on how to use &   #
# configuration.                          #
# See the LICENSE.txt file for more information on the License this software  #
# is distributed under.                         #
#                                #
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.                 #
###############################################################################

use strict;
use warnings;
## added hsh - to support decryption of sentitive data from the DB
use Crypt::CBC;
use MIME::Base64;
use IO::Socket;
use POSIX qw(setsid);
use Socket;

#Read settings from the inprotect.cfg file
use vars qw/%CONFIG/;
load_configs();

#Redirect output to the log file
my $nessuslog = "$CONFIG{'NESSUSLOG'}";
my $serverid = "$CONFIG{'SERVERID'}";
#my $serverid = "";
## added 2/17/07 - hsh - enable encrypted storage of nessus config passwords
my $dbk = "$CONFIG{'DBK'}";
my $dbiv = "$CONFIG{'DBIV'}";

open(LOG,">>$nessuslog") or die "Failed to create $nessuslog: $!\n";
LOG->autoflush(1); # make LOG hot
*STDERR=*LOG;
*STDOUT=*LOG;

print localtime(time)." nessus_scan: Start\n";

#Use Perl modules
use Parallel::ForkManager;
use DBI;

#Declare constants
use constant TRUE => 1;
use constant FALSE => 0;
use constant DEBUG => 0;
use constant MAX_FORKS => 20;

#Declare variables to use
my $outfile = "";
my $outdir = "$CONFIG{'OUTDIR'}";
my $targetfile = "";
my $nessus_cfg = "";
my $failedfile = "";
my $servreg;
my($stopq, $stopid, $stopcmd, $schedid, $username, $email, $schedule_type, $day_of_week, $day_of_month, $time);
my ($stime, $scantime, $sid, $sth_sel, $scanq, $pm);
my ($infield1, $infield2, $infield3, $infield4, $infield5, $infield6, $infield7 );
my ($id, $host, $str, $dbh, $sth, $sth_ins, $sth_upd, $timeout, $catcount );
my ($nessushost, $nessusport, $nessususer, $nessuspassword);
my($time_to_die);
$time_to_die = 0;

# Only user root can run this program 
if ( $> != 0 ) {
    print localtime(time)." nessus_scan: You must be a superuser to start this application\n";
    exit 1;
}

# Check if the program is currently running
if ( CheckRunning("nessus_scan.pl") ) {
    print localtime(time)." nessus_scan: Another instance of the daemon is currently running\n";
    exit 2;
}


# Trap fatal signals
$SIG{INT} = $SIG{QUIT} = $SIG{TERM} = $SIG{HUP} = \&signal_handler;
$SIG{PIPE} = "IGNORE";
$SIG{CHLD} = "IGNORE"; # prevent zombie processes


#Connecting to the database
$dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}","$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
        PrintError => DEBUG,
        RaiseError => 1,
        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");


# Go into an infinite loop and wake up every 5 secs
#until ( $time_to_die ) {

   if(! $dbh->ping) {
      $dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}","$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
         PrintError => DEBUG,
         RaiseError => 1,
         AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
   }

   # get the list of all scanners
   #Select server attributes
   $sth = $dbh->prepare(qq{
      SELECT id, hostname, port, user, password
      FROM nessus_servers
      } );

   $sth->execute;
   my $nessus_servers = $sth->fetchall_hashref('id');

   $sth->finish;

   # prepare some SQL statements for execution inside the loop
   # get the number of scans that are scheduled to be run
   my $sth_ssbr = $dbh->prepare(qq{
      SELECT count(*)
      FROM nessus_scan
      WHERE status = "S"
      AND server_id=?
      } );
   # get the list of scanids that this server needs to scan
   my $sth_sidtr = $dbh->prepare(qq{
      SELECT distinct scan.sched_id 
      FROM nessus_scan scan, nessus_schedule sched
      WHERE scan.status = "S"
      AND scan.server_id=?
      AND scan.sched_id=sched.id
         AND sched.next_scan > 0
      } );

   # loop through the scanner array
   foreach $serverid (keys(%{$nessus_servers})) {
      
      #Get server attributes
      $nessushost = $nessus_servers->{$serverid}->{'hostname'};
      $nessusport = $nessus_servers->{$serverid}->{'port'};
      $nessususer = $nessus_servers->{$serverid}->{'user'};
      $nessuspassword = $nessus_servers->{$serverid}->{'password'};

      $sth_ssbr->execute($serverid);
      ($scanq)=$sth_ssbr->fetchrow_array;
   
      # Open a ForkManager handle limiting the number of children
      # to MAX_FORKS
      $pm = new Parallel::ForkManager($scanq);
   
      $sth_sidtr->execute($serverid);
   
      while(($schedid)=$sth_sidtr->fetchrow_array) {
         $pm->start and next; # Do the fork
         print localtime(time)." nessus_scan: Forked $$ for schedid = $schedid\n";
         my $dbhf;
   
         $dbhf = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}","$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
            PrintError => 0,
            RaiseError => 1,
            AutoCommit => 1 } ) or die("nessus_scan: Failed to connect : $DBI::errstr\n");
   
         # set up some prepared SQL queries for use later
         # decrement current_scans by number of hosts scanned
         $sth_upd=$dbhf->prepare(qq{
           UPDATE nessus_servers
           SET current_scans=current_scans-?
           WHERE id=?
           });
         # set a scan schedule to a running state
         my $sth_updnsr = $dbhf->prepare(qq{
           UPDATE nessus_scan
           SET status="R", start_dttm=now(), process_id=?
           WHERE server_id=?
           AND sched_id=?
           } );

         #Select hosts for this sched to scans to run.
         my $sth_h = $dbhf->prepare(qq{
            SELECT scan.host, scan.sched_id, sched.sid, sched.timeout, 
                   sched.username, sched.next_scan
            FROM nessus_scan scan, nessus_schedule sched
            WHERE scan.status = "S"
            AND scan.server_id=?
            AND scan.sched_id=sched.id
            AND sched.next_scan > 0
            AND scan.sched_id=?
            } );
         $sth_h->execute($serverid,$schedid);
         # create a hash of hashes of the hosts to be scanned
         my (%hosts,$sid,$username,$timeout,$next_scan);
         $sid = $username = $timeout = $scantime = "";
         while(my(@data) = $sth_h->fetchrow_array()) {
            $hosts{ $data[0] } = {
                sched_id => $data[1],
                sid => $data[2],
                timeout => $data[3],
                username => $data[4],
                next_scan => $data[5]
            };
            # assuming that these are the same for all hosts with the
            # same sched_id, they should be
            $sid = $data[2] if ($sid eq "");
            $username = $data[4] if ($username eq "");
            $timeout = $data[3] if ($timeout eq "");
            $scantime = $data[5] if ($scantime eq "");
      #      push @hosts, $data[0];
      #      $sid = $data[2];
      #      $username = $data[4];
      #      $timeout = $data[3];
      #      $next_scan = $data[5];
         }
         $sth_h->finish;
      
         # set them all have a status of running
         # note that the PID is for the child nessus_scan.pl, not the
         # actual nessus scan, we have to get that in the run_nessus
         # subroutine
         $sth_updnsr->execute($$,$serverid,$schedid);
      
         $outfile="${outdir}/nessus_s$$.out";
         $targetfile="${outdir}/target_s$$";
         $nessus_cfg="${outdir}/nessus_s$$.cfg";
   
         unlink $targetfile if -e $targetfile;
         unlink $outfile if -e $outfile;
         unlink $nessus_cfg if -e $nessus_cfg;
      
      #   my $targetinfo = "";
      #   if (-e "$targetfile"){
      #      $targetinfo = ",$host";
      #   } else {
      #      $targetinfo = "$host";
      #   }
         my $targetinfo = join(",",keys %hosts);
         open(TARGET,">>$targetfile") or die "Failed to create $targetfile: $!\n";
         print TARGET "$targetinfo"; 
         close TARGET;
      
         #create_profile ($nessus_cfg, $sid, $dbhf, $schedid, $host, $username, $dbk);
         create_profile ($nessus_cfg, $sid, $dbhf, $schedid, "", $username, $dbk, $dbiv);

         #$dbhf->disconnect;
   
         my $nessusok = run_nessus($outfile, $targetfile, $timeout, $username, 
                                $schedid, keys %hosts);

         if(DEBUG) { print "   run_nessus finished for $schedid\n"; }

         #$dbhf = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}","$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
         #             PrintError => 0,
         #             RaiseError => 1,
         #             AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
   
         # hack for array of hosts
         my $h;
         my $sth_updns=$dbhf->prepare(qq{
            UPDATE nessus_scan
            SET status=?, end_dttm=now()
            WHERE sched_id=?
            AND server_id=?
            AND host=?
            });
         if (!$nessusok) {
            # mark the scan failed
            foreach $h (keys %hosts) {
               $sth_updns->execute("F",$schedid,$serverid,$h);
            }
         } else {
            # Mark scan completed
            foreach $h (keys %hosts) {
               $sth_updns->execute("C",$schedid,$serverid,$h);
            }
         }
         $sth_updns->finish;
      
         # remove entries from live stats table
         my $sth_del=$dbhf->prepare(qq{
            DELETE from nessus_live_stats
            WHERE sid=?
            AND host=?
            });
         # loop through the hash of hosts for this scan
         foreach $h (keys %hosts) {
            $sth_del->execute($schedid,$h);
         }
         $sth_del->finish;
      
         # decrement current_scans by number of hosts scanned
         my $numHosts = keys(%hosts);
         $sth_upd->execute($numHosts, $serverid);
         $sth_upd->finish;
      
         # check if the output file exists and load it into DB
         if(-e $outfile) {
            #Load Nessus results into the db
            load_nessus_results($outfile, $dbhf, $username, 
                                $schedid, $sid, $scantime);
         }
   
         unlink $targetfile if -e $targetfile;
         unlink $outfile if -e $outfile;
         unlink $nessus_cfg if -e $nessus_cfg;

         # Temporary update query to fix formatting error in results table
         my $sth_updnr=$dbhf->prepare(qq{
            UPDATE nessus_results 
            SET service=LEFT(service, LENGTH(service)-1) 
            WHERE RIGHT(service,1) = CHAR(10)
            });
         $sth_updnr->execute;
         $sth_updnr->finish;


         #Close DB Connection
         $dbhf->disconnect or die("nessus_scan: Failed to disconnect : $DBI::errstr\n");
         print localtime(time)." nessus_scan: Fork $$ ending for schedid = $schedid\n";
         $pm->finish;

      };

      #if(DEBUG) { print localtime(time) . "  wait_all_children\n"; }
      #$pm->wait_all_children;
      #if(DEBUG) { print localtime(time) . "  done wait_all_children\n"; }
      #sleep 2;

   } # end of foreach loop for nessus_servers
   $sth_sidtr->finish;
   $sth_ssbr->finish;

#} # end of infinite loop

#Close DB connection
$dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");

print localtime(time)." nessus_scan: Exiting\n";
close LOG;

exit 0;


# Become a daemon 
sub daemonize {
    my ( $pid );
    chdir "/" or die "nessus_scan: Can't chdir to /: $!\n";
    defined($pid = fork) or die "nessus_scan: Can't fork: $!\n";
    exit if $pid;
    setsid or die "nessus_scan: Can't start a new session: $!\n";
    umask 0;
}

# Signal handler for 1,2,3 and 15 
sub signal_handler {
    $time_to_die = 1;
}
## Checks entries in the process table for the program 
## Returns true if entry is found 
sub CheckRunning {
    my ($pgname) = @_;
    my ($exists, $pid);
    $pid = $$;
    open(PROC,"ps -eo '%p %a'|") or die "nessus_scan: Failed to fork : $!\n";

    $exists = 0;
    while(<PROC>) {
       next if /grep|vi|vim|tail|$pid/;
       if ( /$pgname/ ) {
          $exists = 1;
          last;
       }
    }
    close PROC;
    return $exists;
}


sub run_nessus {
#   my ( $outfile, $targetfile, $timeout, $username, $schedid, $host ) = @_;
   my ( $outfile, $targetfile, $timeout, $username, $schedid, @hosts ) = @_;
   my ($cmd, $h);
   my ($sth_del,$sth_ins);
   my ($dbhr);

   $dbhr = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}","$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
                PrintError => 0,
                RaiseError => 1,
                AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");

   # Make sure we dont get dupes in live stats - delete for current schedid/hostid
   $sth_del=$dbhr->prepare(qq{
      DELETE FROM nessus_live_stats
      WHERE sid=$schedid
      AND host =? 
      });

   $sth_ins=$dbhr->prepare(qq{
      INSERT INTO nessus_live_stats
      (sid, host, action, percent, username)
      VALUES (?, ?, "Starting", 0, ?)
      });

   foreach $h (@hosts) {
      $sth_del->execute($h);
      $sth_ins->execute($schedid, $h, $username);
   }

#   print localtime(time)." nessus_scan: Scanning host - $host, User - $username\n";
   foreach $h (@hosts) {
      print localtime(time)." nessus_scan: $schedid -> host - $h, user - $username\n";
   }

   $cmd = qq{$CONFIG{'NESSUSPATH'} -qx $nessushost $nessusport $nessususer $nessuspassword $targetfile $outfile -V -T nbe -c $nessus_cfg};
   my $cmdkill = qq{$CONFIG{'NESSUSPATH'} -qx $nessushost $nessusport };

   for (my $i=0; $i<=length($nessususer)+length($nessuspassword); $i++) {
      $cmdkill=$cmdkill." ";
   }

   $cmdkill=$cmdkill." $targetfile $outfile -V -T nbe -c $nessus_cfg";

   eval {
      local $SIG{ALRM} = sub { die "NessusTimeout\n" };
      alarm $timeout;
#      open(LOGFILE,">>$CONFIG{'NESSUSLOG'}");
      open(PROC,"$cmd |") or die "failed to fork :$!\n";
      # $$ should contain the PID of the child forked to run the scan
      print localtime(time)." nessus_scan: $schedid - forked $$ for scan.\n";
      # TODO update the DB to reflect the proper PID of the scan process
      my (@arr);

      # per the DBI man pages, it's much better to do the prepare
      # outside the loop and just execute inside the loop
      $sth_ins=$dbhr->prepare(qq{
             update nessus_live_stats
             SET action=?, percent=?/?*100
             WHERE sid=?
             AND host=?
             });
      while(<PROC>){
         @arr = split/\|/;
         if(scalar(@hosts) < 10) { # only update the live stats for <10 hosts
#        if ( $arr[2] == $pid ) {
#           $pid_kill = $arr[1];
#        }
            if(is_number($arr[2]) && is_number($arr[3]))
            {
               my $conn_ok = 1;
#
# remove this, check for a problem on the update, saves an extra query
#           $sth_ins=$dbhf->prepare(qq{
#           SELECT host FROM nessus_live_stats
#           WHERE sid=$schedid
#           AND host="$arr[1]"
#           });
#           $sth_ins->execute or $conn_ok=0;
#        
#           if (!$conn_ok)
#           {
#              $dbhf->disconnect or $conn_ok=0;
#
#              $dbhf = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}","$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
#                           PrintError => 0,
#                           RaiseError => 1,
#                           AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
#           }
#
              $sth_ins->execute($arr[0],$arr[2],$arr[3],$schedid,$arr[1]);
            # check for an error updating the DB and re-create connection
            # may lose an update this way, but it will catch up on the next
            # output cycle
              if($sth_ins->err())
              {
                 print localtime(time) . " nessus_scan: DB update problem - " . 
                                        $sth_ins->errstr() . "\n";
                 $dbhr->disconnect or $conn_ok=0;
   
                 $dbhr = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}","$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
                              PrintError => 0,
                              RaiseError => 1,
                              AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
              }

#           print LOGFILE "$arr[0], $arr[1], $arr[2], $arr[3]\n";
#           print  "$arr[0], $arr[1], $arr[2], $arr[3]\n";
            }
         } # end if(@hosts < 10)
      } # end while(<PROC>)
      close PROC;
#      close LOGFILE;
      foreach $h (@hosts) {
         print localtime(time)." nessus_scan: Scan ended - $h, user - $username\n";
      }
      alarm 0;
   };

   $sth_ins->finish;
   $sth_del->finish;
   $dbhr->disconnect;

   if ($@ && $@ eq "NessusTimeout\n" ) {
      warn "Nessus Scan timed out for $host\n";

      `ps auxwww \| grep \"$cmdkill\" \| perl -pe '(\$_)=/root {3,9}(\\d+)/;\`kill \$_\`'`;
      return FALSE;
   } else {
      return TRUE;
   };
}

sub create_profile {

   my ($nessus_cfg, $sid, $dbh, $schedid, $host, $username, $dbk, $dbiv) = @_;

   my ($sth_selp, $sth_selp1, $sth_selp2, $nessus_id, $nessus_value);


# don't care if this host was scanned before
#   #Find if the host was tested before
#   $sth_selp=$dbh->prepare(qq{
#      SELECT count(host)
#      FROM nessus_results
#      WHERE username="$username"
#          AND host="$host"
#          AND sched_id=$schedid
#          AND sid=$sid
#      });
#   $sth_selp->execute;
#
#   my($scancount)=$sth_selp->fetchrow_array;
#   my($scantime)=0;
#
#   if($scancount>0) {
#      #Find when the host was last time tested
#      $sth_selp=$dbh->prepare(qq{
#         SELECT scantime
#         FROM nessus_results
#         WHERE username="$username"
#             AND sched_id=$schedid
#             AND sid=$sid
#             AND host="$host"
#         ORDER BY scantime DESC
#         LIMIT 1
#         });
#      $sth_selp->execute;
#      ($scantime)=$sth_selp->fetchrow_array;
#   } #end if

   open(PROFILE,">$nessus_cfg") or die "Failed to create $nessus_cfg: $!\n";

   $sth_selp=$dbh->prepare(qq{
      SELECT autoenable 
      FROM nessus_settings
      WHERE id=$sid
      });
   $sth_selp->execute;

   my ($autoenable)=$sth_selp->fetchrow_array;

   $sth_selp=$dbh->prepare(qq{
      SELECT nessus_id, value 
      FROM nessus_settings_preferences
      WHERE category IS NULL 
           AND sid=$sid
      } );
   $sth_selp->execute;

   while (($nessus_id, $nessus_value)=$sth_selp->fetchrow_array ) {
      # Exclude null nessus_value records
      if ($nessus_value) {
         print PROFILE "$nessus_id = $nessus_value\n";
      }
      else {
         print PROFILE "$nessus_id\n";
      }
   }

   $sth_selp=$dbh->prepare(qq{
      SELECT t1.id, t1.enabled 
      FROM nessus_settings_plugins as t1, 
           plugins as t2
      WHERE t1.id=t2.id 
           AND t2.category ='scanner' 
           AND t1.sid=$sid 
      ORDER BY id
      } );
   $sth_selp->execute;

   print PROFILE "begin(SCANNER_SET)\n";

   while (($nessus_id, $nessus_value)=$sth_selp->fetchrow_array ) {
      if ($nessus_value eq "N") {
         $nessus_value="no";
      }
      else {
         $nessus_value="yes";
      }
      print PROFILE " $nessus_id = $nessus_value\n";
   }
   print PROFILE "end(SCANNER_SET)\n\n";

   $sth_selp=$dbh->prepare(qq{
      SELECT nessus_id, value 
      FROM nessus_settings_preferences
      WHERE category='SERVER_PREFS' 
          AND sid=$sid
      } );
   $sth_selp->execute;
   print PROFILE "begin(SERVER_PREFS)\n";

   while (($nessus_id, $nessus_value)=$sth_selp->fetchrow_array ) {
      print PROFILE " $nessus_id = $nessus_value\n";
   }

   print PROFILE "end(SERVER_PREFS)\n\n";

        #now get the plugin preferences
   print PROFILE "begin(PLUGINS_PREFS)\n";

## modified 2/17/07 - hsh - enable storage of sensitive passwords
## in an encrypted form (Blowfish)
   $sth_selp=$dbh->prepare(qq{
      SELECT nessus_id, value 
      FROM nessus_settings_preferences
      WHERE category='PLUGINS_PREFS' 
          AND sid=$sid
      } );
   $sth_selp->execute;

## initialize the encryption routines
   my $cipher = Crypt::CBC->new( {'key'             => $dbk,
                               'cipher'          => 'Blowfish',
                               'header'          => 'randomiv'
   #                            'iv'              => $dbiv,
# deprecated                            'regenerate_key'  => 0,   # default true
   #                            'padding'         => 'standard',
   #                            'literal_key'     => 0, # default false
   #                            'prepend_iv'      => 0
                              });

   while (($nessus_id, $nessus_value)=$sth_selp->fetchrow_array ) {
## check for encrypted values and decrypt accordingly
      if($nessus_value =~ /^ENC\{(.*)\}/) {
         my $ciphertext = decode_base64($1);
         $nessus_value = $cipher->decrypt($ciphertext);
      }
      print PROFILE " $nessus_id = $nessus_value\n";
   }

   print PROFILE "end(PLUGINS_PREFS)\n\n";

   print PROFILE "\n\nbegin(PLUGIN_SET)\n";

   if ($autoenable eq "C") {
      $sth_selp=$dbh->prepare(qq{
         SELECT t1.cid, t1.status 
         FROM nessus_settings_category t1, 
              nessus_category t2
         WHERE t2.name<>"scanner"
             AND t2.id=t1.cid
             AND t1.sid=$sid
         ORDER BY t2.id
         });
      $sth_selp->execute;
      while(my($catid, $catstatus)=$sth_selp->fetchrow_array) {
#         if ($catstatus==5 and $scancount>0) {
         if ($catstatus==5) {
            $sth_selp1=$dbh->prepare(qq{
               SELECT count( t2.scriptid )
               FROM nessus_plugins t1, 
                    nessus_results t2
               WHERE t2.username = "$username"
                   AND t1.id = t2.scriptid
                   AND t1.category=$catid
                   AND t2.sid=$sid
                   AND t2.sched_id=$schedid
               });
#                   AND t2.host = "$host"
            $sth_selp1->execute;

            ($catcount)=$sth_selp1->fetchrow_array;

            if ($catcount>0) {
               #Scan for existing good plugins within this category
               $sth_selp2=$dbh->prepare(qq{
                  SELECT distinct t1.scriptid 
                  FROM nessus_results t1, 
                       nessus_plugins t2
                  WHERE t1.scriptid=t2.id
                      AND t2.category=$catid
                      AND t1.sid=$sid
                      AND t1.sched_id=$schedid
                      AND t2.created <=$scantime
                      AND t2.deleted is NULL
                      OR t1.scriptid=t2.id
                      AND t2.category=$catid
                      AND t1.sid=$sid
                      AND t1.sched_id=$schedid
                      AND t1.host="$host"
                      AND t2.modified <=$scantime
                      AND t2.deleted is NULL
                  ORDER BY t1.scriptid
                  });
#                      AND t1.host="$host"
               $sth_selp2->execute;
               while(($nessus_id)=$sth_selp2->fetchrow_array) {
                  print PROFILE " $nessus_id = yes\n";
               }

               #Scan new/modified plugins
               $sth_selp2=$dbh->prepare(qq{
                  SELECT id
                  FROM nessus_plugins
                  WHERE modified > $scantime
                      AND category=$catid
                      AND deleted is NULL
                      AND category=$catid
                      AND deleted is NULL
                  ORDER by id
                  });
#                      OR created > $scantime
               $sth_selp2->execute;
               while(($nessus_id)=$sth_selp2->fetchrow_array) {
                  print PROFILE " $nessus_id = yes\n";
               }#end loop

               #Do not scan other plugins in this category
               $sth_selp2=$dbh->prepare(qq{
                  SELECT DISTINCT nessus_plugins.id
                  FROM nessus_plugins
                  LEFT JOIN nessus_results 
                  ON nessus_plugins.id = nessus_results.scriptid
                      AND nessus_results.sched_id=$schedid
                      AND nessus_results.sid=$sid
                  WHERE nessus_results.scriptid is null
                      AND nessus_plugins.category=$catid
                      AND nessus_plugins.deleted is NULL
                  ORDER BY nessus_plugins.id
                  });
#                      AND nessus_results.host="$host"
               $sth_selp2->execute;
               while(($nessus_id)=$sth_selp2->fetchrow_array){
                  print PROFILE " $nessus_id = no\n";
               }
            }
            else {
               $sth_selp2=$dbh->prepare(qq{
                  SELECT id
                  FROM nessus_plugins
                  WHERE modified > $scantime
                      AND category=$catid
                      AND deleted is NULL
                      AND category=$catid
                      AND deleted is NULL
                  ORDER by id
               });
#                      OR created > $scantime
               $sth_selp2->execute;
               while(my($diff_plugin)=$sth_selp2->fetchrow_array) {
                  print PROFILE " $diff_plugin = yes\n";
               }#end loop
                  $sth_selp2=$dbh->prepare(qq{
                     SELECT id
                     FROM nessus_plugins
                     WHERE modified <= $scantime
                         AND category=$catid
                         AND deleted is NULL
                         AND category=$catid
                         AND deleted is NULL
                     ORDER by id
                  });
#                         OR created <= $scantime
                  $sth_selp2->execute;
                  while(my($disable_plugin)=$sth_selp2->fetchrow_array){
                     print PROFILE " $disable_plugin = no\n";
                  }
               }

               $sth_selp1->finish;
            } #end if
            else {

            $sth_selp2=$dbh->prepare(qq{
               SELECT t1.id, t1.enabled 
               FROM nessus_settings_plugins t1, 
                    nessus_plugins t2
               WHERE t1.id=t2.id 
                   AND t2.category =$catid 
                   AND t1.sid=$sid 
               ORDER BY id
               } );
            $sth_selp2->execute;

            while (($nessus_id, $nessus_value)=$sth_selp2->fetchrow_array ) {
               if ($nessus_value eq "N") {
                  $nessus_value="no";
               }
               else {
                  $nessus_value="yes";
               }
               print PROFILE " $nessus_id = $nessus_value\n";
            } #end loop
         } #end if

         $sth_selp2->finish;
      } #end loop
   } #end if
   elsif ($autoenable eq "F") {
      $sth_selp=$dbh->prepare(qq{
         SELECT t1.fid, t1.status 
         FROM nessus_settings_family t1, 
              nessus_family t2
         WHERE t2.name<>"Port scanners"
             AND t2.id=t1.fid
             AND t1.sid=$sid
         ORDER BY t2.id
         });
      $sth_selp->execute;
      while(my($famid, $famstatus)=$sth_selp->fetchrow_array) {
#         if ($famstatus==5 and $scancount>0) {
         if ($famstatus==5) {
            $sth_selp1=$dbh->prepare(qq{
               SELECT count( t2.scriptid )
               FROM nessus_plugins t1, 
                    nessus_results t2
               WHERE t2.username = "$username"
                   AND t1.id = t2.scriptid
                   AND t1.family=$famid
                   AND t2.sid=$sid
                   AND t2.sched_id=$schedid
               });
#                   AND t2.host = "$host"
            $sth_selp1->execute;

            ($catcount)=$sth_selp1->fetchrow_array;

            if ($catcount>0) {
               #Scan for existing good plugins within this category
               $sth_selp2=$dbh->prepare(qq{
                  SELECT distinct t1.scriptid 
                  FROM nessus_results t1, 
                       nessus_plugins t2
                  WHERE t1.scriptid=t2.id
                      AND t2.family=$famid
                      AND t1.sid=$sid
                      AND t1.sched_id=$schedid
                      AND t2.created <=$scantime
                      AND t2.deleted is NULL
                      OR t1.scriptid=t2.id
                      AND t2.family=$famid
                      AND t1.sid=$sid
                      AND t1.sched_id=$schedid
                      AND t1.host="$host"
                      AND t2.modified <=$scantime
                      AND t2.deleted is NULL
                  ORDER BY t1.scriptid
                  });
#                      AND t1.host="$host"
               $sth_selp2->execute;

               while(($nessus_id)=$sth_selp2->fetchrow_array) {
                  print PROFILE " $nessus_id = yes\n";
               }

               #Scan new/modified plugins
               $sth_selp2=$dbh->prepare(qq{
                  SELECT id
                  FROM nessus_plugins
                  WHERE
                      family=$famid
                      AND deleted is NULL
                      AND family=$famid
                      AND  deleted is NULL
                  ORDER by id
                  });
#                  WHERE modified > $scantime
#                      AND family=$famid
#                      OR created > $scantime
               $sth_selp2->execute;

               while(($nessus_id)=$sth_selp2->fetchrow_array) {
                  print PROFILE " $nessus_id = yes\n";
               }#end loop

               #Do not scan other plugins in this category
               $sth_selp2=$dbh->prepare(qq{
                  SELECT DISTINCT nessus_plugins.id
                  FROM nessus_plugins
                  LEFT JOIN nessus_results 
                  ON nessus_plugins.id = nessus_results.scriptid
                      AND nessus_results.sched_id=$schedid
                      AND nessus_results.sid=$sid
                  WHERE nessus_results.scriptid is null
                      AND nessus_plugins.family=$famid
                      AND nessus_plugins.deleted is NULL
                  ORDER BY nessus_plugins.id
                  });
#                      AND nessus_results.host="$host"
               $sth_selp2->execute;

               while(($nessus_id)=$sth_selp2->fetchrow_array){
                  print PROFILE " $nessus_id = no\n";
               }
            }
            else {
               $sth_selp2=$dbh->prepare(qq{
                  SELECT id
                  FROM nessus_plugins
                  WHERE 
                      family=$famid
                      AND deleted is NULL
                  ORDER by id
               });
#                  WHERE modified > $scantime
#                      AND family=$famid
#                      OR created > $scantime
               $sth_selp2->execute;
               while(my($diff_plugin)=$sth_selp2->fetchrow_array) {
                  print PROFILE " $diff_plugin = yes\n";
               }#end loop
               $sth_selp2=$dbh->prepare(qq{
                  SELECT id
                  FROM nessus_plugins
                  WHERE
                      family=$famid
                      AND deleted is NULL
                  ORDER by id
               });
#                  WHERE modified <= $scantime
#                      AND family=$famid
#                      AND deleted is NULL
#                      OR created <= $scantime
#                      AND family=$famid
               $sth_selp2->execute;
               while(my($disable_plugin)=$sth_selp2->fetchrow_array){
                  print PROFILE " $disable_plugin = no\n";
               }
            }

            $sth_selp1->finish;
         } #end if
         else {

            $sth_selp2=$dbh->prepare(qq{
               SELECT t1.id, t1.enabled 
               FROM nessus_settings_plugins t1, 
                    nessus_plugins t2
               WHERE t1.id=t2.id 
                   AND t2.family =$famid 
                   AND t1.sid=$sid 
               ORDER BY id
            } );
            $sth_selp2->execute;

            while (($nessus_id, $nessus_value)=$sth_selp2->fetchrow_array ) {
               if ($nessus_value eq "N") {
                  $nessus_value="no";
               }
               else {
                  $nessus_value="yes";
               }
               print PROFILE " $nessus_id = $nessus_value\n";
            } #end loop
         } #end if

         $sth_selp2->finish;
      } #end loop
   }
   else {
      $sth_selp=$dbh->prepare(qq{
         SELECT t1.id, t1.enabled 
         FROM nessus_settings_plugins as t1, 
              plugins as t2
         WHERE t1.id=t2.id 
             AND t2.category <>'scanner' 
             AND t1.sid=$sid 
         ORDER BY id
         } );
      $sth_selp->execute;
      #print PROFILE "\n\nbegin(PLUGIN_SET)\n";

      while (($nessus_id, $nessus_value)=$sth_selp->fetchrow_array ) {
         if ($nessus_value eq "N") {
            $nessus_value="no";
         }
         else {
            $nessus_value="yes";
         }
         print PROFILE " $nessus_id = $nessus_value\n";
      } #end loop

   } #end if


   print PROFILE "end(PLUGIN_SET)\n\n";
   $sth_selp->finish;


}


sub load_configs
{
   my $configfile="/etc/inprotect/inprotect.cfg";
   my $noconfig=0;
   open(CONF,"<$configfile") || $noconfig++;
   my @CONFILE=<CONF>;
   close(CONF);

   if ($noconfig) { 
      print localtime(time)." nessus_scan: No inprotect.cfg file found.\n"; 
   }
   foreach my $line (@CONFILE)
   {
      $line =~ s/\#.*$//;
      chomp($line);
      $line =~ s/\s+$//;
      $line =~ s/^\s+//;
      if ($line eq "") { next; }
      my @temp=split(/=/,$line,2);
      if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
   }
   return;
}


sub load_nessus_results {

   my ( $outfile, $dbhf, $username, $schedid, $sid, $scantime) = @_;
   my ( $dupecheck, $sqlc1, $sqlc2, $stmt, $insql, $sth1, $sth2);


   # Open file for input
   if (DEBUG) {
      print localtime(time)." nessus_scan: Importing results $outfile\n";
   }
   open(INPUT,"<$outfile")||die("Can't open report file $outfile");

   # prepare some SQL queries to allow the DB to do an execution plan
   # to speed up the processing
   # statement for inserting a new result into the DB
   $insql = qq{insert into nessus_results
               (username, sched_id, sid, scantime, scantype, 
                domain, host, service, scriptid, risk, msg)
               values ( '$username', '$schedid', '$sid', '$scantime',
                        'A', ?, ?, ?, ?, ?, ?)};
   $stmt = $dbhf->prepare($insql);

   # statement 1 for checking DB for dupes (infield5 ne " ")
   $sqlc1 = qq{select count(*) 
               from nessus_results 
               where username = '$username' 
                  and sched_id = '$schedid' 
                  and sid = '$sid' 
                  and scantime = '$scantime'
                  and scantype = 'A'
                  and domain = ?
                  and host = ?
                  and service = ?
                  and scriptid = ?
                  and risk = ?
                  and msg = ?};
   $sth1 = $dbhf->prepare($sqlc1);

   # statement 2 for checking DB for dupes (infield5 == " ")
   $sqlc2 = qq{select count(*) 
               from nessus_results 
               where username = '$username' 
                  and sched_id = '$schedid' 
                  and sid = '$sid' 
                  and scantime = '$scantime'
                  and scantype = 'A'
                  and domain = ?
                  and host = ?
                  and service = ?
                  and scriptid = ?
                  and risk = ? };
   $sth2 = $dbhf->prepare($sqlc2);

   # loop through input file and insert into table
   while (<INPUT>)
   {
      # Initialize field values each time to ensure the are clear
      #
      $infield1 = " ";
      $infield2 = " ";
      $infield3 = " ";
      $infield4 = " ";
      $infield5 = " ";
      $infield6 = " ";
      $infield7 = " ";


      # Break out fields from input file
      #
      ($infield1,$infield2,$infield3,$infield4,$infield5,$infield6,$infield7)=split(/\|/,$_);

      # Some nessus results only have 4 fields - Fix null values here
      if (!defined($infield5)) {
         $infield5 = " ";
      }
      if (!defined($infield6)) {
         $infield6 = " ";
      }
      if (!defined($infield7)) {
         $infield7 = " ";
      }
      
      # Translate ; to \n for newline
      #
      $infield7 =~ s/;/\\n/g;
      #remove \n from start
      $infield7 =~ s/^(\\n)+//;
      $infield7 =~ s/(\\n)+$//;
      $infield7 =~ s/\n+$//;

      # Translate " to \"
      #
      #$infield7 =~ s/\"/'/gm;

      # Convert "infield6 factor" to infield6 value HIGH/MEDIUM/LOW
      #
      $infield6='7';
      $infield6='1' if ($infield7 =~ m/Risk [fF]actor\s*:\s*(\\n)*Serious/s);
      $infield6='1' if ($infield7 =~ m/Risk [fF]actor\s*:\s*(\\n)*Critical/s);
      $infield6='2' if ($infield7 =~ m/Risk [fF]actor\s*:\s*(\\n)*High/s);
      $infield6='3' if ($infield7 =~ m/Risk [fF]actor\s*:\s*(\\n)*Medium/s);
      $infield6='4' if ($infield7 =~ m/Risk [fF]actor\s*:\s*(\\n)*Medium\/Low/s);
      $infield6='5' if ($infield7 =~ m/Risk [fF]actor\s*:\s*(\\n)*Low\/Medium/s);
      $infield6='6' if ($infield7 =~ m/Risk [fF]actor\s*:\s*(\\n)*Low/s);
      $infield6='7' if ($infield7 =~ m/Risk [fF]actor\s*:\s*(\\n)*Info/s);
      $infield6='7' if ($infield7 =~ m/Risk [fF]actor\s*:\s*(\\n)*[nN]one/s);

      # Remove "infield6 factor" from $infield7 field
      # Using ` vs / due to existing / in field
      #
      $infield7=~ s/Risk [fF]actor\s*:\s*(\\n)*Serious((\\n)+| \/ |$)//s;
      $infield7=~ s/Risk [fF]actor\s*:\s*(\\n)*Critical((\\n)+| \/ |$)//s;
      $infield7=~ s/Risk [fF]actor\s*:\s*(\\n)*High((\\n)+| \/ |$)//s;
      $infield7=~ s/Risk [fF]actor\s*:\s*(\\n)*Medium((\\n)+| \/ |$)//s;
      $infield7=~ s/Risk [fF]actor\s*:\s*(\\n)*Medium\/Low((\\n)+| \/ |$)//s;
      $infield7=~ s/Risk [fF]actor\s*:\s*(\\n)*Low\/Medium((\\n)+| \/ |$)//s;
      $infield7=~ s/Risk [fF]actor\s*:\s*(\\n)*Low((\\n)+| \/ |$)//s;
      $infield7=~ s/Risk [fF]actor\s*:\s*(\\n)*Info((\\n)+| \/ |$)//s;
      $infield7=~ s/Risk [fF]actor\s*:\s*(\\n)*[nN]one to High((\\n)+|(\s)+| \/ |$)//;
      $infield7=~ s/Risk [fF]actor\s*:\s*(\\n)*[nN]one((\\n)+| \/ |$)//s;

      #remove \n from end
      $infield7 =~ s/(\\n)+$//;
      $infield7 =~ s/\n+$//;
      # convert \\n to \n as the prepare statement takes care of properly
      # escaping specials when they are inserted into the DB, otherwise
      # we end up with the literal \\n in the field.
      $infield7 =~ s/\\n/\n/g;
      # stip out any \r at the end as the prepare also takes care of 
      # converting the \n into the correct thing
      $infield7 =~ s/\\r$//g;
      $infield4 = htmlspecialchars($infield4);
      $infield7 = htmlspecialchars($infield7);

      # Insert values into appropriate table
      #
      if ($infield1 eq "results") {
         # CHECK FOR DUPLICATE ENTRIES
#         $sql = qq{select count(*) 
#                   from nessus_results 
#                   where username = '$username' 
#                      and sched_id = '$schedid' 
#                      and sid = '$sid' 
#                      and scantime = '$scantime'
#                      and scantype = 'A'
#                      and domain = '$infield2'
#                      and host = '$infield3'
#                      and service = '$infield4'
#                      and scriptid = '$infield5'
#                      and risk = '$infield6'};
#
#         if ($infield5 ne " ") {
#            $sql .= " and msg = '$infield7'";
#         }
#         $sth = $dbhf->prepare($sql);
#         $sth->execute;

         if ($infield5 ne " ") {
            $sth1->execute($infield2, $infield2, $infield4, $infield5,
                           $infield6, $infield7);
            ($dupecheck) = $sth1->fetchrow_array;
         } else {
            $sth2->execute($infield2, $infield2, $infield4, $infield5,
                           $infield6);
            ($dupecheck) = $sth2->fetchrow_array;
         }
      
         if ($dupecheck == 0) {
            $stmt->execute( $infield2, $infield3, $infield4, $infield5,
                            $infield6, $infield7);
#            $dbhf->do("insert into nessus_results
#                          (username, sched_id, sid, scantime, scantype, 
#                           domain, host, service, scriptid, risk, msg)
#                       values ('$username', $schedid, $sid, '$scantime',
#                               'A','$infield2','$infield3','$infield4',
#                               '$infield5','$infield6',\"$infield7\")
#                      ");
         } else {
            if (DEBUG) {
               print localtime(time).
                  " nessus_scan: Duplicate result " . 
                  "$infield2|$infield3|$infield4|$infield5|$infield6".
                  "|$infield7\n";
            }
         }  
#         $sth->finish;
      }
   }
   $stmt->finish;
   $sth1->finish;
   $sth2->finish;
   close INPUT;
}

sub is_number{
   my($n)=@_;
   return ($n=~/^\d+$/)
}

# Function to convert certain chars to HTML entities - same as PHP's htmlspecialchars function

sub htmlspecialchars
{
   my $tmpSTRmsg = $_[0];

   $tmpSTRmsg =~ s/&/&amp;/g;
   $tmpSTRmsg =~ s/\'/&#039;/g;
   $tmpSTRmsg =~ s/\"/&quot;/g;
   $tmpSTRmsg =~ s/</&lt;/g;
   $tmpSTRmsg =~ s/>/&gt;/g;

   return $tmpSTRmsg;
}
