#!/usr/bin/perl
#
# $Id: sched.pl,v 1.13 2006/08/10 17:19:39 airwofl Exp $
#
################################################################################
#				     Inprotect				      #
#-----------------------------------------------------------------------------#
# Copyright (C) 2002 - 2004 Inprotect.com				      #
#									      #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of version 2 of the GNU General Public License as published #
# by the Free Software Foundation.					      #
#									      #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.								      #
#									      #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc.,     #
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA			      #
#									      #
# Contact Information:							      #
# Alex Zimin (alex@inprotect.com)					      #
# http://www.inprotect.com						      #
###############################################################################
# See the README.txt and/or help files for more information on how to use &   #
# configuration.							      #
# See the LICENSE.txt file for more information on the License this software  #
# is distributed under.							      #
#									      #
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.				      #
###############################################################################



use strict;
use warnings;
use MIME::Lite;
use Date::Calc qw( Days_in_Month Day_of_Week Add_Delta_YMD);
use DBI;
use Socket;
use POSIX qw(setsid :sys_wait_h);
use IO::Socket;

#Declare constants
use constant TRUE => 1;
use constant FALSE => 0;
use constant DEBUG => 0;

#Declare vars
my ($timenow,$wday,$year, $mon,$mday,$now);
my($hostip);
my($startip, $endip, $startip_n, $endip_n, $ip1, $ip2, $ip3, $ip4, $ip5, $ip6, $ip7, $ip8, $oct1, $oct2, $oct3, $oct4, $var1, $var2, $scanip, $start_n, $end_n, $nip, $is_submitted, $sth_check);
my($noncomplete,$stopq, $stopid, $stopcmd, $schedid, $username, $email, $schedule_type, $day_of_week, $day_of_month, $time);
my ($stime, $scantime, $sid, $sth_sel, $pm);
my ($infield1, $infield2, $infield3, $infield4, $infield5, $infield6, $infield7, $insertid);
my ($id, $host, $str, $dbh, $sth, $sth_ins, $sth_upd, $timeout);
my($time_to_die);
$time_to_die = 0;


#Read settings from the inprotect.cfg file
use vars qw/%CONFIG/;
load_configs();

# how often to check the status of the nessus scanner
my $checkInterval = "$CONFIG{'CHECKINTERVAL'}";

# debug output flag, 0 = off, 1 = on
my $debug = 1; 

# hash for keeping track of child PIDs
my %sched_pids;

#Redirect output to the log file
my $nessuslog = "$CONFIG{'SCHEDLOG'}";
open(LOG,">>$nessuslog") or die "Failed to create $nessuslog: $!\n";
*STDERR=*LOG;
*STDOUT=*LOG;
LOG->autoflush(1); # make LOG hot, causing no output delays
print localtime(time)." sched: Start\n";

# Only user root can run this program 
if ( $> != 0 ) {
    print localtime(time)." sched: You must be a superuser to start this application\n";
    exit 1;
}


# Check if the program is currently running
if ( CheckRunning("sched.pl") ) {
    print localtime(time)." sched: Another instance of the daemon is currently running\n";
    exit 2;
}


# Become a daemon
print localtime(time)." sched: Become a daemon\n";
&daemonize;


# Trap fatal signals
$SIG{INT} = $SIG{QUIT} = $SIG{TERM} = $SIG{HUP} = \&signal_handler;
$SIG{PIPE} = "IGNORE";
$SIG{CHLD} = 'IGNORE';


#Connecting to the database
$dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}","$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
                        PrintError => $debug,
                        RaiseError => 1,
                        AutoCommit => 1 } ) or die(localtime(time) .
                                 " sched: Failed to connect : $DBI::errstr\n");


#Read some more config data 
my @fields = qw{mailfrom inprotect_url scanNotificationCC replyto};
my %settings;
$sth=$dbh->prepare(qq{
                       SELECT settingValue
                       FROM inprotect_settings
                       WHERE settingName = ?
                   });
foreach(@fields) {
   $sth->execute($_);
   ($settings{$_}) = $sth->fetchrow_array(); 
}

#Prepare database at startup
start_up($dbh);

# Go into an infinite loop and wake up every 5 secs
until ( $time_to_die ) {

    # check for any scans to stop (migrated from nessus_run.pl)
    check_scans_to_stop($dbh, %settings);

    #Process completed scans
    complete_scans($dbh,%settings);

    #Getting system time
    my ($sec,$min,$hour,$yday,$isdst);
    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
#my $now;
#my $timenow;
    $mon++;
    $now = sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
    $timenow = sprintf("%02d%02d%02d", $hour, $min, $sec);

    # check the scanners to make sure that they are alive and well
    check_connect($dbh,$checkInterval);

    #checking for the new unscheduled scans
    $sth = $dbh->prepare(qq{
                             SELECT  id, username, schedule_type, 
                                     day_of_week, day_of_month, time
                             FROM nessus_schedule
                             WHERE status is NULL
                         } );
    $sth->execute;
    
    # schedule them to be run
    while(($schedid, $username, $schedule_type, $day_of_week, 
           $day_of_month, $time)=$sth->fetchrow_array) {
        $stime=substr($time,0,2).substr($time,3,2)."00";
        schedule_scan ($schedid, $username, $schedule_type, $stime,
                       $day_of_week, $day_of_month);
    };


    #Select scheduled scans to run.
    $sth = $dbh->prepare(qq{
                             SELECT h.hostip, h.sched_id
                             FROM nessus_hosts h, nessus_schedule s
                             WHERE h.sched_id = s.id
                             AND s.status = "S"
                             AND s.next_scan <= "$now"
                         } );
    $sth->execute;

    # Insert into nessus_scan table
    # 1. Split IP range into individual IP's before inserting into nessus_scan
    # 2. Assign 

    # prep the query to insert the data into the nessus_scan table
    #   the "server" value defaults to 0 per the DB schema
    my ( $sth_ins_nessus_scan );
    $sth_ins_nessus_scan = $dbh->prepare(qq{
                                             INSERT INTO nessus_scan
                                               ( sched_id, host, status, 
                                                 submit_dttm )
                                             VALUES(?,?,?,now())
                                           } );
    # prep the query to check if the host for a sched_id has already been
    # put in the nessus_scan table to be picked up by nessus_run.pl
    $sth_check=$dbh->prepare(qq{
                                 SELECT count(*) from nessus_scan
                                 WHERE sched_id=?
                                    AND host=?
                               });

    my ($scans_to_schedule) = 0;
    # loop through all the hosts and sched_ids and take appropriate action
    while(($hostip, $sid) = $sth->fetchrow_array ){
        print localtime(time)." sched: Checking scanid=$sid, hostip=$hostip\n";

        # check to see if the $hostip is a range of addresses and split
        # them up accordingly
        ($startip, $endip)=split(/-/,$hostip);

        if (!defined($endip)) {
            $endip=$startip;
        }
        $startip_n=inet_aton($startip);
        $endip_n=inet_aton($endip);
        ($ip1, $ip2, $ip3, $ip4)=unpack("CCCC", $startip_n);
        ($ip5, $ip6, $ip7, $ip8)=unpack("CCCC", $endip_n);
        $start_n=$ip1*16777216+$ip2*65536+$ip3*256+$ip4;
        $end_n=$ip5*16777216+$ip6*65536+$ip7*256+$ip8;

        for ($nip=$start_n; $nip<=$end_n; $nip++) {
            $oct1=int($nip/16777216);
            $var1=$nip%16777216;
            $oct2=int($var1/65536);
            $var2=$var1%65536;
            $oct3=int($var2/256);
            $oct4=$var2%256;
		      # Skip Network and Broadcast addresses
            next if ( $oct4 == 0 or $oct4 == 255 );
            $scanip="$oct1.$oct2.$oct3.$oct4";

		      # Insert individual IP's into nessus_scan
            #  check if it's there first
            $sth_check->execute($sid,$scanip);
            ($is_submitted)=$sth_check->fetchrow_array;
            if($is_submitted==0) {
                # nope, so let's insert it
                $sth_ins_nessus_scan->execute($sid, $scanip, "S");
                $scans_to_schedule++;
            }
        }
    }
    if($scans_to_schedule > 0) {
        print localtime(time)." sched: Scheduled $scans_to_schedule scans to be run.\n";
    }

    # set the status as "R"unning in the nessus_schedule table
    $sth_upd=$dbh->prepare(qq{
                               UPDATE nessus_schedule
                               SET status="R"
                               WHERE status="S"
                               AND next_scan<= "$now"
                           });
    $sth_upd->execute;

    # clean up the DB handles
    $sth_ins_nessus_scan->finish;
    $sth_check->finish;
    $sth_upd->finish;
    $sth->finish;

    # see if the online scan engines have any open scan slots
    $sth_sel=$dbh->prepare(qq{
                               SELECT SUM(max_scans-current_scans)
                               FROM nessus_servers
                               WHERE checkin_time+interval 300 second>now()
                               AND max_scans>current_scans
                               AND status = "A"
                           });

    $sth_sel->execute;
    my $available_server_rsc = $sth_sel->fetchrow_array;
    if (!defined($available_server_rsc)){
        $available_server_rsc=0;
    }

    # Get number of scheduled scans to run
    $sth_sel=$dbh->prepare(qq{
                               select count(*)
                               from nessus_scan
                               where status='S' and server_id=0
                           });
    $sth_sel->execute;
    my $number_of_ips_to_scan = $sth_sel->fetchrow_array;
    if (!defined($number_of_ips_to_scan)){
        $number_of_ips_to_scan=0;
    }

    # Set $number_of_scans to minimum of available scan slots or
    #  actual scans to perform
    my $number_of_scans;
    if($available_server_rsc > $number_of_ips_to_scan) {
        $number_of_scans = $number_of_ips_to_scan;
    } else {
        $number_of_scans = $available_server_rsc;
    }

    # if there are any scans to be run, get the data for them
    # note that this is only looking for scans that haven't been assigned
    # to a server yet (server_id=0)
    if ($number_of_scans > 0) {
        $sth_sel=$dbh->prepare(qq{
                                   SELECT t1.host, t1.sched_id, t2.username
                                   FROM nessus_scan t1, nessus_schedule t2
                                   WHERE t1.status="S" AND t1.server_id=0
                                   AND t2.id=t1.sched_id
                                   ORDER BY submit_dttm
                                   LIMIT $number_of_scans
                               });

        $sth_sel->execute;

        my($sth_sel1, $serv_id, $user_name);

        # prep the query to get the scanner id that can scan the selected IP
        $sth_sel1=$dbh->prepare(qq{
                                    SELECT t3.nessusid
                                    FROM nessus_zone_ranges t2,
                                         nessus_server_zones t3, 
                                         nessus_servers t4, 
                                         nessus_user_zones t5
                                    WHERE inet_aton( ? )  >= t2.range_start
                                    AND inet_aton( ? )  <= t2.range_end
                                    AND t3.zoneid = t2.zoneid
                                    AND t3.nessusid=t4.id
                                    AND t4.current_scans<t4.max_scans
                                    AND t4.checkin_time+interval 300 second>now()
                                    AND t5.zid=t3.zoneid
                                    AND t5.username=?
                                    AND t4.status='A'
                                    ORDER by t4.max_scans-t4.current_scans
                                    DESC
                                    LIMIT 1
                                  });
        # prep the update to the nessus_scan table to assign a server to
        #  the entry
        $sth_upd=$dbh->prepare(qq{
                                   UPDATE nessus_scan
                                   SET server_id=?
                                   WHERE host=?
                                      AND sched_id=?
                                  });
        my $scans_scheduled = 0;
        my %scheduled_scan_data;
        # loop through all the scans yet to be assigned to a server
        while (my($current_ip, $sched_id, $user_name) = 
                 $sth_sel->fetchrow_array) {
           # add data to a hash for use in the notification email
           # find a scanner that is configured to scan this IP for this user
           $sth_sel1->execute($current_ip,$current_ip,$user_name);
           ($serv_id)=$sth_sel1->fetchrow_array;
	        if(not $serv_id) {
		        $serv_id = 0;
	        }
           # found one, increment the current_scans count on this scanner
           # and assign the IP to it in nessus_scan
           if($serv_id > 0) {
		        print localtime(time)." scheduling host: $current_ip 
                       for schedule $sched_id on server $serv_id\n";
              # update the nessus_scan table
              $sth_upd->execute($serv_id, $current_ip, $sched_id);
              # increment the current_scans value for this scanner
              my $sth_upd2=$dbh->prepare(qq{
                                         UPDATE nessus_servers
                                         SET current_scans=current_scans+1
                                         WHERE id=$serv_id
                                       });
              $sth_upd2->execute;
              $scans_scheduled++;
              push(@{$scheduled_scan_data{$sched_id}}, $current_ip);
		      print localtime(time)." scheduled host: $current_ip " .
                       "for schedule $sched_id on server $serv_id\n";
              $number_of_scans=$number_of_scans-1;
              # close up DB handle
              $sth_upd2->finish;
           } # end if()
       } # end while()

       # fork/exec the other perl program to actually run the scans
       if($scans_scheduled > 0) { # we scheduled some scans
          my $pid = runit("/usr/bin/nessus_scan.pl");
          # may not be implemented in the Win32 API for PERL, so let's just
          # skip this for now... <sigh>
          #sleep 2;
          #my $s = kill 0, $pid;
          #if(not $s) {
          #   print localtime(time) . " sched.pl: Couldn't fork nessus_scan.pl\n";
          #} else {
             $sched_pids{$pid} = $pid;
          #}
       }

       #Send start scan email notifications
       $sth=$dbh->prepare(qq{
                              SELECT t2.pn_email, t2.pn_uname, 
                                     t1.next_scan
                              FROM nessus_schedule t1, users t2
                              WHERE t1.username=t2.pn_uname
                              AND t1.id = ?
                          });
       for my $sched_id (keys %scheduled_scan_data ) {
          $sth->execute($sched_id);
          my($mailto, $susername, $sscantime)=$sth->fetchrow_array;
          if(defined($settings{'scanNotificationCC'})) {
             $mailto .= ", " . $settings{'scanNotificationCC'};
          }
          scan_started_email($mailto, $susername, $sched_id, $sscantime, 
                             $settings{'mailfrom'}, $settings{'replyto'} );
		    print localtime(time)." scheduled ". 
                                scalar(@{$scheduled_scan_data{$sched_id}}) . 
                                " hosts to be scanned for $sched_id.\n";
       }
       # close DB handles
       $sth->finish;
       $sth_sel1->finish;
    }

    # close DB handles
    $sth_sel->finish;
    $sth_upd->finish;
    $sth->finish;

    # go to sleep for 5 sec before we start it all over again
    sleep 5;
}

#Close DB connection
$dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");


print localtime(time)." sched: Finish\n";
close LOG;

exit 0;

# subroutine to fork/exec the nessus_scan.pl program to actually
# run the scans and load the results
sub runit {
   my $which=shift;

FORK: {  
   if (my $pid=fork) {
      return $pid;
      # this is the parent, so return the pid
      # everything below here is either the child or a very major 
      # system failure
   } elsif (defined $pid) {
      my @estuff=($which);
#      open (O,">>$nessuslog") or print "Can't write to $nessuslog $!";
#      print O "Running " . @estuff;
#      close O;
      exec @estuff or print localtime(time) . "nessus_scan: Can't exec process\n";;
      # shouldn't reach this unless exec fails
#      open (O,">>$nessuslog") or print "Can't write to $nessuslog $!";
#      print O "Couldn't exec\n";
#      close O;
      exit 0;
      # exit, NOT return.  We're the child process
   } elsif ($! == "EAGAIN")  {
      sleep 3;
      redo FORK;
   } else {
#      open (O,">>$nessuslog") or print "Can't write to $nessuslog $!";
#      print O "Can't fork $!\n";
#      close O;
      print localtime(time) . "nessus_scan: Could not fork $!";
      return 0;
   }
}

}

sub schedule_scan {
    my ($schedid, $username, $schedule_type, $stime, $day_of_week, $day_of_month) =@_;
    my ($dy, $dm, $dd);
    $dy=0;
    $dm=0;
    $dd=0;

    if ($schedule_type eq "N") {
    } elsif ($schedule_type eq "D") {
       if ($stime<$timenow) {
		     #Update next scan datetime
           $dd=1;
       };
    } elsif ($schedule_type eq "W") {
       if ($day_of_week<$wday) {
          $dd=7-$wday+$day_of_week;
       } elsif ($day_of_week==$wday && $stime<$timenow) {
          $dd=7;
       } elsif ($day_of_week>$wday) {
          $dd=$day_of_week-$wday;
       };
    } elsif ($schedule_type eq "M") {
	    #Getting max numbers of days in the month
       my ($daysm, $mydom);
       $mydom=$day_of_month;
       $daysm=Days_in_Month($year, $mon);
       my ($nyear, $nmon);
       if ($mon<12) {
          $nmon=$mon+1;
          $nyear=$year;
       } else {
          $nmon=1;
          $nyear=$year+1;
       };

       my ($daysnm);
       $daysnm=Days_in_Month($nyear, $nmon);
       if ($daysm < $day_of_month) {
          $mydom=$daysm;
       };
       if ($mydom < $mday) {
          $dd=$daysm-$mday;
          if ($day_of_month gt $daysnm) {
             $dd=$dd+$daysnm;
          } else {
             $dd=$dd+$day_of_month;
          };
       } elsif ($mydom == $mday && $stime lt $timenow){
          $dd=$daysm-$mday;
          if ($day_of_month gt $daysnm) {
             $dd=$dd+$daysnm;
          } else {
             $dd=$dd+$day_of_month;
          };
       } elsif ($mydom > $mday) {
          $dd=$mydom-$mday;
       };
    };
    my $nextscan;

    if ($schedule_type eq "N") {
       $nextscan = "$now";
    } else {
       my ($syear, $smonth, $sday)=Add_Delta_YMD($year,$mon,$mday,$dy,$dm,$dd);
       if ($smonth<10) {
            $smonth="0".$smonth;
       };

       if ($sday<10) {
            $sday="0".$sday;
       };

        $nextscan = "$syear$smonth$sday$stime";
    }

    $sth_upd = $dbh->prepare(qq{
                                 UPDATE nessus_schedule
                                 SET next_scan='$nextscan', status='S'
                                 WHERE username='$username' AND id=$schedid
                             } );

    $sth_upd->execute;
    $sth_upd->finish;
} #end schedule_scan()


sub load_configs
{
    my $configfile="/usr/local/etc/inprotect.cfg";
    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." sched: No inprotect.cfg file found.\n"; }
    foreach my $line (@CONFILE)
    {
        $line =~ s/\#.*$//;
               chomp($line);
        $line =~ s/\s+$//;
               $line =~ s/^\s+//;
                      if ($line eq "") { next; }
                      my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
} #end load_configs()


sub email_results {
    my ( $to, $username, $scantime, $dbhf, $schedid, $from, $replyto, $resulturl) = @_;
    my ( $msg, $html_data, $sth_sel );

    $sth_sel=$dbh->prepare(qq{
                               SELECT COUNT(DISTINCT host)
                               FROM nessus_results
                               WHERE scantime='$scantime'
                               AND username='$username'
                               AND sched_id=$schedid
                           } );
    $sth_sel->execute;
    my ($hostcount)=$sth_sel->fetchrow_array;
    $sth_sel=$dbh->prepare(qq{
                               SELECT COUNT(risk) as rcount, risk
                               FROM nessus_results
                               WHERE scantime='$scantime'
                               AND sched_id=$schedid
                               AND username='$username'
                               GROUP BY risk
                           } );
    $sth_sel->execute;

    my $riskres = "";
    my $riskcounttotal = 0;

    while (my($riskcount, $risklevel)= $sth_sel->fetchrow_array) {
       my $rt = riskText($risklevel);
       $riskres .= "
Risk Level:     $rt ($risklevel) - $riskcount";
       $riskcounttotal=$riskcounttotal+$riskcount;
    }


    $sth_sel=$dbh->prepare(qq{
                               SELECT COUNT(DISTINCT scantime) as scount
                               FROM nessus_results
                               WHERE sched_id=$schedid
                               AND username='$username'
                           } );
    $sth_sel->execute;

    my($scancount)=$sth_sel->fetchrow_array;

    my $prevscanmsg = "";


    if ($scancount>1) {
    $sth_sel=$dbh->prepare(qq{
                               SELECT distinct scantime as prevscan 
                               FROM nessus_results
                               WHERE sched_id=$schedid AND username='$username' 
                               ORDER BY scantime DESC 
                               LIMIT 1,1
                           } );
        $sth_sel->execute;
        my($prevscan)=$sth_sel->fetchrow_array;

        $sth_sel=$dbh->prepare(qq{
                                   SELECT count(distinct host) 
                                   FROM nessus_results
                                   WHERE scantime='$prevscan' AND 
                                         username='$username' AND 
                                         sched_id=$schedid
                               } );
        $sth_sel->execute;
        my ($prevhostcount)=$sth_sel->fetchrow_array;

        $sth_sel=$dbh->prepare(qq{
                                   SELECT count(risk) as rcount, risk 
                                   FROM nessus_results
                                   WHERE scantime='$prevscan' AND 
                                         sched_id=$schedid AND 
                                         username='$username' 
                                   GROUP BY risk
                               } );
        $sth_sel->execute;

        my $prevriskres = "";
        my $prevriskcounttotal = 0;

        while (my($prevriskcount, $prevrisklevel)= $sth_sel->fetchrow_array) {
            my $rt = riskText($prevrisklevel);
            $prevriskres .= "
Risk Level:         $rt ($prevrisklevel) - $prevriskcount";
            $prevriskcounttotal=$prevriskcounttotal+$prevriskcount;
        }
        $prevscanmsg="
Previous summary:   Scanned $prevhostcount system(s)
Vulns found:        $prevriskcounttotal
$prevriskres";
    }

    $sth_sel=$dbh->prepare(qq{
				SELECT name 
            FROM nessus_schedule 
            WHERE id=$schedid
			  } );
    $sth_sel->execute;
    my $schedname = $sth_sel->fetchrow_array;

    $msg = MIME::Lite->new(
               To      =>"$to",
               From    =>"$from",
               'Reply-To' => $replyto,
               Subject =>"$CONFIG{'EMAILSUBJECT'}",
               Data    =>"
Dear $username, 

Your scheduled vulnerability scan has finished. You can access the scan results here: 

$resulturl/results.php?scantime=$scantime&schedid=$schedid&scantype=A

Schedule:       $schedid - $schedname

Scan summary:   Scanned $hostcount system(s)
Vulns found:    $riskcounttotal
$riskres

----------------------------------------------------------------------

$prevscanmsg
"
           );
    $msg->send();
    $sth_sel->finish;
}# end email_results()

# convert the risk level to the text value
sub riskText {
    my ( $rl ) = @_;
    for ($rl) {
       /^1/ && do { return "Serious"; last ; };
       /^2/ && do { return "High"; last ; };
       /^3/ && do { return "Medium"; last ; };
       /^4/ && do { return "Medium"; last ; };
       /^5/ && do { return "Low"; last ; };
       /^6/ && do { return "Low"; last ; };
       /^7/ && do { return "Info"; last ; };
       return "Unknown";
    }
} # end riskText()

sub scan_started_email {
    my ( $to, $username, $schedrec, $scantime, $from, $replyto ) = @_; 
    my ( $msg, $html_data );
    my ( $sth_get );

    $sth_get=$dbh->prepare(qq{
                                SELECT name 
                                FROM nessus_schedule 
                                WHERE id=$schedrec
                          } );
    $sth_get->execute;
    my $schedname = $sth_get->fetchrow_array;

    $msg = MIME::Lite->new(
               To      => "$to",
               From    => "$from",
               'Reply-To'  => "$replyto", 
               Subject => "Vulnerability scan started.",
               Data    => "Dear $username,

Your scheduled vulnerability scan has started.

Schedule name:		$schedrec - $schedname
"
           );
    $msg->send();
    # release DB statement handle
    $sth_get->finish;
} #end scan_started_email()

# Become a daemon 
sub daemonize {
    my ( $pid );
    chdir "/" or die "sched: Can't chdir to /: $!\n";
    defined($pid = fork) or die "Can't fork: $!\n";
    exit if $pid;
    setsid or die "sched: Can't start a new session: $!\n";
    umask 0;
} # end daemonize()

# Signal handler for 1,2,3 and 15 so we can cleanly exit
sub signal_handler {
    $time_to_die = 1;
    print localtime(time)." sched: Received exit signal, exiting.\n";
} #end signal_handler()

## Checks entries in the process table for the program 
## Returns true if entry is found 
sub CheckRunning {
    my ($pgname) = @_;
    my ($exists, $pid);
    $pid = $$;
    open(PROC,"ps -eo '%p %a'|") or die "Failed to fork : $!\n";

    $exists = 0;
    while(<PROC>) {
        next if /grep|vi|vim|tail|$pid/;
        if ( /$pgname/ ) {
                $exists = 1;
                last;
            }
        }
    close PROC;
    return $exists;
} # end CheckRunning()


##Mark scans completed
sub complete_scans {

    my ( $dbh,%settings) = @_;

    # Get a list of schedules
    $sth_sel=$dbh->prepare(qq{
                               SELECT distinct sched_id
                               FROM nessus_scan
                           });
    $sth_sel->execute;

    # For each schedule
    while(($schedid)=$sth_sel->fetchrow_array){
	    # For the current schedule, how many scans are RUNNING or WAITING to run?
       $sth=$dbh->prepare(qq{
                           SELECT count(*)
                           FROM nessus_scan
                           WHERE sched_id=$schedid
                           AND NOT (status="C" OR status = "F")
                       });
        $sth->execute;
        ($noncomplete)=$sth->fetchrow_array;

	     # If there are no scans for current schedule running, delete 
        # all scans for current schedid
        if ($noncomplete==0) {
            $sth=$dbh->prepare(qq{
                                   DELETE
                                   FROM nessus_scan
                                   WHERE sched_id=$schedid
                                   AND NOT (status="R" OR status IS NULL)
                               });
            $sth->execute;

		      # List scans as "complete" for NOW or ONCE ONLY schedule items
            $sth=$dbh->prepare(qq{
                                   UPDATE nessus_schedule
                                   SET status="C"
                                   WHERE (id=$schedid
                                   AND schedule_type="N")
                                   OR (id=$schedid
                                   AND schedule_type="O")
                               });
            $sth->execute;

		      # For daily/weekly/monthly, set status to NULL for completed scans
            $sth=$dbh->prepare(qq{
                                   UPDATE nessus_schedule
                                   SET status=null
                                   WHERE id=$schedid
                                   AND schedule_type="D"
                                   OR id=$schedid
                                   AND schedule_type="W"
                                   OR id=$schedid
                                   AND schedule_type="M"
                               });
            $sth->execute;

		      # Get details of who owns the completed scan
            $sth=$dbh->prepare(qq{
                                   SELECT t1.username, t2.pn_email, t1.next_scan
                                   FROM nessus_schedule t1, users t2
                                   WHERE t1.id=$schedid
                                   AND t1.username=t2.pn_uname
                               });
            $sth->execute;
            my($user_sm, $email_sm, $scan_sm)=$sth->fetchrow_array;

		      # Send them a report
            my $mailto = $email_sm;
            if(defined($settings{'scanNotificationCC'})) {
               $mailto .= ", " . $settings{'scanNotificationCC'};
            }
            email_results($mailto, $user_sm, $scan_sm, $dbh, $schedid, $settings{'mailfrom'}, $settings{'replyto'}, $settings{'inprotect_url'});
        }
    }
    # release DB statement handles
    $sth->finish;
    $sth_sel->finish;
} # end complete_scans()

# Check if the Nessus scanners are up
sub check_connect {

   my ( $dbh, $interval ) = @_;


   my $sth1;
   my($serverid,$n_hostname,$n_port);
   my($socket);
   # get list of scanners that haven't been checked in 300+ seconds
   # removed check for status <> "C" - hsh 10/9/07
   my $sth=$dbh->prepare(qq{
                          SELECT id, hostname, port
                          FROM nessus_servers
                          WHERE ((checkin_time + interval $interval second) < now())
                          OR checkin_time is null
                      });
   $sth->execute;

   # set the status to "Checking"
   # removed this, the check should be fast enough that this isn't necessary
#   $sth1=$dbh->prepare(qq{
#                           UPDATE nessus_servers
#                           SET status="C"
#                           WHERE (checkin_time + interval $interval second < now()
#                           AND status <> "C")
#                           OR checkin_time is null
#                       });
#   $sth1->execute;
#   $sth1->finish;

   #Check status of the Nessus servers loop
   while(($serverid,$n_hostname, $n_port)=$sth->fetchrow_array) {
      my $connect = -1;
      #Check if the server is alive
      #print localtime(time)." sched: Checking Nessus scanner status - $n_hostname:$n_port\n";
      #$socket = IO::Socket::INET->new("$n_hostname:$n_port") or $connect=0;
      $socket = IO::Socket::INET->new("$n_hostname:$n_port") or $connect = 0;  

      #If server is dead
      my $peer;
      if($connect == 0) {
         undef($peer);
      } else {
         $peer = $socket->connected();
         close($socket);
      }
      if(defined($peer) && $connect != 0) {
         # was able to connect to scanner, update checkin time and set
         # status to "A"vailable
         print localtime(time) . " sched: Nessus scanner ONLINE - " .
               $n_hostname . ":" . $n_port . "\n";
         $sth1=$dbh->prepare(qq{
                               UPDATE nessus_servers
                               SET status="A",
                               checkin_time=now()
                               WHERE id=$serverid
                               });
         $sth1->execute;
         $sth1->finish;
      } else {
         print localtime(time) . " sched: Nessus scanner OFFLINE - " .
               $n_hostname . ":" . $n_port . "\n";
         # update the status to "N"ot online
         $sth1=$dbh->prepare(qq{
                               UPDATE nessus_servers
                               SET status="N",
                               checkin_time=now(),
                               current_scans=0
                               WHERE id=$serverid
                               });
         $sth1->execute;
   
         # unassign any scans that were assigned to it
         # Reschedule scan jobs so that another server might pick them up   
         $sth1=$dbh->prepare(qq{
                               UPDATE nessus_scan
                               SET server_id=0, status="S", 
                               start_dttm=NULL, end_dttm=NULL
                               WHERE server_id=$serverid
                               AND (status="R"
                               OR status="S")
                               });
         $sth1->execute;
         $sth1->finish;
      }
      #close $socket;
   } #End of while loop
   # release DB statement handle to free resources
   $sth->finish;
} # end check_connect()

#This should take care of abnormal possible things
sub start_up {

    my ( $dbh) = @_;
    my ($sth);
    #Force nessus servers check upon startup by seting them to "U"nknown
    $sth=$dbh->prepare(qq{
                           UPDATE nessus_servers
                           SET status="U",
                               checkin_time=now()-interval 301 second
                       });
    $sth->execute;
    # release DB statement handle to free resources
    $sth->finish;
} # end start_up()


sub check_scans_to_stop {
   my ($dbh, %settings) = @_;

   # get the list of all scanners
   #Select server attributes
   my $sth = $dbh->prepare(qq{
      SELECT id, hostname
      FROM nessus_servers
      } );

   $sth->execute;
   my $nessus_servers = $sth->fetchall_hashref('id');

   $sth->finish;

   # prepare some SQL statements for execution inside the loop
   #Check for scans to stop
   my $sth_stssel = $dbh->prepare(qq{
      SELECT distinct process_id
      FROM nessus_scan
      WHERE status = "P"
      AND server_id=?
      });

   # loop through the scanner array
   foreach my $serverid (keys(%{$nessus_servers})) {
      $sth_stssel->execute($serverid);
      my $stopid;
      $sth_stssel->bind_columns(\$stopid);
      while($sth_stssel->fetch) {
         if(DEBUG) { print localtime(time)." sched: asked to kill $stopid\n"; }
         # since nessus_scan.pl spawns children to do the actual nessus client
         # invocation, we can get the nessus client PID by looking for any
         # process with a PPID=stopid
         my $pid = `ps -o pid= --ppid $stopid`;
         chomp($pid);
         # get the PPID for stopid so we can use it to clean up that process
         my $ppid = `ps -o ppid= --pid $stopid`;
         chomp($ppid);
         while($pid ne "") {
            if(DEBUG) { print localtime(time)." sched: killing $pid\n"; }
            kill 7, $pid;
            $pid = `ps -o pid= --ppid $stopid`;
            chomp($pid);
         } 
         # clean up the children that we've killed
         if(DEBUG) { print localtime(time)." sched: cleaning up child $ppid\n"; }
         #do {
            $pid = waitpid($ppid,0);
         #} until $pid > 0;
      } #end of loop

      sleep 1;

   } # end of foreach loop for nessus_servers
   $sth_stssel->finish;

} # end check_scans_to_stop()
