#!/bin/sh

web="/var/www/html"
if [ -e "REV" ]; then
   REV=`cat REV`
else
   REV="0.80.0"
fi

install_web()
{
echo "Setting up Inprotect Web Console interface"
echo "=========================================="
echo $Xn "Inprotect webroot path [/var/www/html] " $Xc
read webroot < /dev/tty
test -n "$webroot" && web="$webroot"

test -d "$web" || {
        echo $Xn "$web does not exist - do you want to create it ? [y] " $Xc
        read answer < /dev/tty
        test "$answer" = "n" && {echo "Can not continue then. Bye" && exit}
        mkdir -p $web || {
                echo "Could not create $web"
                exit
                }
        }

test -d "/usr/local" || {
	mkdir -p /usr/local || {
		echo "Could not create /usr/local"
		exit
	}
}

test -d "/usr/local/bin" || {
	mkdir -p /usr/local/bin || {
		echo "Could not create /usr/local/bin"
		exit
	}
}

test -d "/usr/local/etc" || {
	mkdir -p /usr/local/etc || {
		echo "Could not create /usr/local/etc"
		exit
	}
}

cp console/html/* $web -rf
cp scanner/*.pl /usr/local/bin -rf
cp console/scripts/sched.pl /usr/local/bin -rf
cp console/scripts/inprotect.cfg /usr/local/etc -rf

echo "/usr/local/bin/inprotect_reset.pl" >> /etc/rc.d/rc.local
echo "/usr/local/bin/sched.pl" >> /etc/rc.d/rc.local

crontab -l | grep -F -v -f console/crontab_web | grep -v ^\# > /tmp/cron.tmp
cat console/crontab_web >> /tmp/cron.tmp
crontab /tmp/cron.tmp
rm /tmp/cron.tmp -rf

crontab -l | grep -F -v -f scanner/crontab | grep -v ^\# > /tmp/cron.tmp
cat scanner/crontab >> /tmp/cron.tmp
crontab /tmp/cron.tmp
rm /tmp/cron.tmp -rf

echo "Web console setup finished."
echo "Please review settings in the files:"
echo "  - /usr/local/etc/inprotect.cfg"
echo "  - $web/config.php"

}


install_db(){
echo "Setting up Inprotect database"
echo "============================="

FAIL=1
while [ "$FAIL" -eq "1" ];
do
    read -p "Is this a clean install or an upgrade from 0.22.5 [C=clean, U=upgrade, default = C]? " inst_type
    if [ $inst_type = "U" -o $inst_type = "u" ]; then
       DBFILE="console/sql/upgrade_0.22.5-$REV.sql"
    else
       DBFILE="console/sql/clean_install_inprotect_$REV.sql"
    fi
    echo "using file $DBFILE"
    read -p "Enter the name of the database to use for the InProtect installation [inprotect]: " mysql_db
    if [ $mysql_db = "" ]; then
       mysql_db="inprotect"
    fi
    read -p "Enter the name of the MySql user to use for setting up the DB [root]: " mysql_user
    if [ $mysql_user = "" ]; then
       mysql_user="root"
    fi
    echo "You will be prompted twice for the MySql user password."
#    read -s -p "Enter the password for MySql user $mysql_user: " mysql_pass
#    if [ " $mysql_pass" = " " ]; then
#        if [ $inst_type = "C" ]; then
#           mysql -u $mysql_user -e "CREATE DATABASE IF NOT EXISTS $mysql_db;"
#        else
#           mysqldump -u $mysql_user $mysql_db > $mysql_db-backup.sql
#           echo "Backup of existing database created in $mysql_db-backup.sql"
#        fi
#		mysql -u $mysql_user $mysql_db < $DBFILE
#		FAIL=0
#    else
#        # test the password
#		mysql -u $mysql_user -p$mysql_pass -e "show databases;" >> /dev/null
#		if [ $? -eq 0 ]; then
           if [ $inst_type = "C" -o $inst_type = "c" ]; then
              mysqldump -u $mysql_user -p $mysql_db > $mysql_db-backup.sql
              echo "Backup of existing database created in $mysql_db-backup.sql"
           #else if [ $inst_type = "U" -o $inst_type = "u" ]; then
           else
               mysql -u $mysql_user -p -e "CREATE DATABASE IF NOT EXISTS $mysql_db;"
           fi
		   mysql -u $mysql_user -p  $mysql_db < $DBFILE
           if [ $? -eq 0 ]; then
			   FAIL=0
		   else
               read -p "There was a problem setting up the DB, try again? (y/n) " try_again
               if [ $try_again = "y" ]; then
		          FAIL=1
               else
                  FAIL=0
               fi
           fi
#        else
#           read -p "There was a problem setting up the DB, try again? (y/n) " try_again
#           if [ $try_again -eq "y" ]; then
#              FAIL=1
#           else
#              FAIL=0
#           fi
#		fi
#    fi
done

echo "Database setup finished."

}


echo "Inprotect $REV installation"
echo "======================"
echo "Please run this installation as root user"
echo "1 - Install Web Console interface only"
echo "2 - Install Database only"
echo "3 - install Web Console & Database components"
echo "Q - Quit without installation"
echo
echo "N.B. To install a Scanner, all you need is Nessus and required"
echo "components installed on a scanner host - see INSTALL file (part I) for"
echo "more details."
echo
echo $Xn "Make your selection [1-3] " $Xc
read answer < /dev/tty

case $answer in 
	1)
		install_web
		;;
	2)	
		install_db
		;;
	3)
		install_web
		install_db
		;;
	Q)	;;
	*)
		echo "Wrong selection was made, please re-run the installation script"
		;;
esac
