<?php
/*
 * Common functions that can be used by any PHP page.
 * It keeps the code cleaner
 */

// sanitize the form input
function sanitize($input) {
   $output = $tmp = "";
   if(get_magic_quotes_gpc()) {
      $tmp = stripslashes(trim($input));
   } else {
      $tmp = $input;
   }
   $output = mysql_real_escape_string($tmp);
   $output = htmlspecialchars( $output , ENT_QUOTES);
   return $output;
}

function dispSQLError($err, $errNum) {
   echo "<table class='error'>
         <tr><th> $errNum SQL Errors<th></tr>\n";
   foreach($err as $e) {
      echo "<tr><td>$e</td></tr>\n";
   }
   echo "</table>\n";
}

//function getUserRoles ($user, $dbconn) {
function getUserRoles ($user) {
   global $dbconn;
   $finalroles = array();
   // get users roles, used for access control decisions
   //  user roles are a union of the users individually assigned roles
   //  and any roles assigned based on user group membership
   // get the roles from group memebership for the user
   $query = "SELECT r.roleName
             FROM users as u
             LEFT JOIN user_groups as ug ON u.pn_uname = ug.pn_uname
             LEFT JOIN group_roles as gr ON ug.gid = gr.gid
             LEFT JOIN roles as r ON gr.roleID = r.roleID
             WHERE u.pn_uname = '$user'";
   $groles = $dbconn->GetArray($query);
   if($groles === false) {
      $errMsg[] = "SQL Error getting user groups: " . $dbconn->ErrorMsg();
      $error = 1;
   } else {
      foreach($groles as $role) {
         $finalroles[$role['roleName']] = TRUE;
      }
   }

   // now get any directly assigned roles
   $query = "SELECT r.roleName
             FROM users as u
             LEFT JOIN user_roles as ur ON u.pn_uname = ur.pn_uname
             LEFT JOIN roles as r ON ur.roleID = r.roleID
             WHERE u.pn_uname = '$user'";
   $uroles = $dbconn->GetArray($query);
   if($uroles === false) {
      $errMsg[] = "SQL Error getting users roles: " . $dbconn->ErrorMsg();
      $error++;
   } else {
      foreach($uroles as $role) {
         $finalroles[$role['roleName']] = TRUE;
      }
   }
   
   // now set the other roles to FALSE in the array
   $query = "SELECT roleName
             FROM roles";
   $roles = $dbconn->GetArray($query);
   if($roles === false) {
   }  else {
      foreach($roles as $role) {
         $finalroles[$role['roleName']] = (isset($finalroles[$role['roleName']])) ? TRUE : FALSE;
      }
   }
   return $finalroles;
}

function inArray ($tmpItem, $tmpArray)
{
   $found=0;

   for ($i=0; $i<count($tmpArray); $i++)
   {
      if ($tmpItem==$tmpArray[$i])
      {
         $found=1;
         break;
      }
   }

   if ($found) { return 1; }
   else { return 0; }
}

/*
 * Function to check password quality
 */
function checkPass($pass) {
   $lAlpha = '#[a-z]#'; //"abcdefghijklmnopqrstuvwxyz";
   $uAlpha = '#[A-Z]#'; //"ABCDEFGHIJKLMNOPQRSTUVWXYZ";
   $numbers = '#[0-9]#'; //'0123456789';
   $specials = '/[!"$%^&*()`{}\[\]:@~;\'#<>?,.\/\\-=_+\|]/';
   // moved to config.php
//   $minLength = 8;
//   $maxLength = 64;
//   $minAlpha = 1;
//   $minLower = 1;
//   $minUpper = 1;
//   $minSpecial = 1;
//   $minNumber = 1;
   global $minLength, $maxLength, $minAlpha, $minLower, $minUpper;
   global $minSpecial, $minNumber;
   $isValid = true;

   if($minLength > 0) {
      if(strlen($pass) < $minLength) { return "fail minLength"; }
      //echo "passed minLength\n";
   }
   if($maxLength > 0) {
      if(strlen($pass) > $maxLength) { return "fail maxLength"; }
      //echo "passed maxLength\n";
   }
   if($minAlpha > 0) {
      preg_match('#[a-z]#',$pass,$lmatches);
      preg_match('#[A-Z]#',$pass,$umatches);
      if((count($lmatches) + count($umatches)) < $minAlpha) { return "fail minAlpha"; }
      //echo "passed minAlpha\n";
      //if(strspn($pass,$lAlpha.$uAlpha) < $minAlpha) { return false; }
   }
   if($minSpecial > 0) {
      preg_match($specials,$pass,$matches);
      if(count($matches) < $minSpecial) { return "fail minSpecial"; }
      //echo "passed minSpecial\n";
      //if(strspn($pass,$specials) < $minSpecial) { return false; }
   }
   if($minNumber > 0) {
      preg_match('#[0-9]#',$pass,$matches);
      if(count($matches) < $minNumber) { return "fail minNumber"; }
      //echo "passed minNumber\n";
      //if(strspn($pass,$numbers) < $minNumber) { return false; }
   }
   if($minUpper > 0) {
      preg_match('#[A-Z]#',$pass,$matches);
      if(count($matches) < $minUpper) { return "fail minUpper"; }
      //echo "passed minUpper\n";
      //if(strspn($pass,$lAlpha) < $minUpper) { return false; }
   }
   if($minLower > 0) {
      preg_match('#[a-z]#',$pass,$matches);
      if(count($matches) < $minLower) { return "fail minLower"; }
      //echo "passed minLower\n";
      //if(strspn($pass,$uAlpha) < $minLower) { return false; }
   }

   return "pass"; 
}

/*
 * Convert risk number to text
 */
function getrisk($risk) {

    switch ($risk) {
    case 1:
            $risk="Serious";
        break;
    case 2:
        $risk="High";
        break;
    case 3:
        $risk="Medium";
        break;
    case 4:
        $risk="Medium/Low";
        break;
    case 5:
        $risk="Low/Medium";
        break;
    case 6:
        $risk="Low";
        break;
    case 7:
        $risk="Info";
        break;
    }

    return $risk;
}

function hostLink($host, $ip, $type = "name", $edit = FALSE) {

   if($edit) {
      $edit = "op=edit";
   } else {
      $edit = "op=view";
   }
   
   // special case for unknown hostnames
   if(strstr($host,"unknown")) {
      $ip = substr($host,8);
      $url = "<a href='hosts.php?ip=$ip&op=add' onClick=\"popup('hosts.php?ip=$ip&op=add','Add Host');return false;\" title='Add host'>Unknown</a>";
      return $url;
   }

   // blank hostname and ip
   if($host == "" && $ip == "") {
      $url = "<a href='hosts.php?$edit' onClick=\"popup('hosts.php?$edit','Lookup Host');return false;\" title='Lookup host'>All</a>";
      return $url;
   }

   if($type == "name") {
      $url = "<a href='hosts.php?host=$host&ip=$ip&$edit' onClick=\"popup('hosts.php?host=$host&ip=$ip&$edit','Lookup Host');return false;\" title='Lookup host'>$host</a>";
   } else { // IP address
      // do special check for multiple IP addresses, comma separated
      $ips = explode(",",$ip);
      foreach($ips as $v) {
         $url[] = "<a href='hosts.php?host=$host&ip=$v&$edit' onClick=\"popup('hosts.php?host=$host&ip=$v&$edit','Lookup IP');return false;\" title='Lookup IP'>$v</a> ";
      }
      $url = implode($url,",");
   }
   return $url;
}

function validateip($ip) {
    $retip=0;

    if (ereg('^([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])$',$ip,$r)) {
        $ip1=intval($r[1])*16777216+intval($r[2])*65536+intval($r[3])*256+intval($r[4]);
        if ($ip1<>0) {
            $retip=1;
        }
    } elseif (ereg('^([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])-([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])$',$ip,$r)) {

        $ip1=intval($r[1])*16777216+intval($r[2])*65536+intval($r[3])*256+intval($r[4]);
        $ip2=intval($r[5])*16777216+intval($r[6])*65536+intval($r[7])*256+intval($r[8]);

        if ($ip2>$ip1 and $ip1<>0) {
            $retip=1;
        }
    }
    return($retip);
}

// function to check if the submitted IP address is in an allowed Network Zone
// that the user has rights to scan.
function inrange($ip, $dbconn) {
    $check=0;
    // check for single IP address, and split it into the octets
    if (ereg('^([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])$',$ip,$r)) {

        $ip1=intval($r[1])*16777216+intval($r[2])*65536+intval($r[3])*256+intval($r[4]);
        $ip2=0;

        if ($ip1<>0) {
            $check=1;
        }
    // check for range of IP address, and split them into the two sets of
    // octets (for ip1 and ip2)
    } elseif (ereg('^([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])-([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2,3}|2[0-4][0-9]|25[0-5])$',$ip,$r)) {

        $ip1=intval($r[1])*16777216+intval($r[2])*65536+intval($r[3])*256+intval($r[4]);
        $ip2=intval($r[5])*16777216+intval($r[6])*65536+intval($r[7])*256+intval($r[8]);

        if ($ip2>$ip1 and $ip1<>0) {
            $check=1;
        }
    }

    if ($check==1) {
        // get the username
        $uname=htmlspecialchars(mysql_escape_string(trim($_SESSION['user'])), ENT_QUOTES);
        $found=0;

        // get the ranges for all the Network Zones that the user has
        // access to so we can check to see if the IP(s) are in one of them.
        $query="SELECT range_start, range_end 
                FROM nessus_zone_ranges 
                INNER JOIN nessus_user_zones 
                   ON nessus_user_zones.zid=nessus_zone_ranges.zoneid 
                WHERE nessus_user_zones.username='$uname'";

        $result=$dbconn->execute($query);

        while(!$result->EOF) {

            list($range_start, $range_end)=$result->fields;

            $rip1=0;
            $rip2=0;

            // single IP, range that is a single IP as well
            if ($ip2==0 and $range_start==$range_end) {
                $rip1=$range_start;

                if ($rip1==$ip1) {
                    $found=1;
                    break;
                }
            }

            // if this is a valid range (start is before end)
            if ($range_start<$range_end) {

                $rip1=$range_start;
                $rip2=$range_end;

                // for a single IP, see if it's between the start and the end
                // (inclusive)
                // or for a range, the starting IP and ending IP have to both
                // be between the start and end (inclusive)
                if (($ip1>=$rip1 and $ip1<=$rip2 and $ip2==0) 
                     or ($ip1>=$rip1 and $ip2<=$rip2 and $ip2<>0)) {
                    $found=1;
                    break;
                }
            }
            $result->MoveNext();
        }
        return($found);
    } else {
        return(0);
    }
}

function dateformat($scantime) {
   if (strlen($scantime) != 14) {
      return $scantime;
   }
   $rval=substr($scantime,4,2)."/"
         .substr($scantime,6,2)."/"
         .substr($scantime,0,4)." "
         .substr($scantime,8,2).":"
         .substr($scantime,10,2).":"
         .substr($scantime,12,2);
   return $rval;

}

function addException($ip,$res,$plugin,$schedid,$hostname) {
   return "<br><a href='editException.php?op=new&ip=$ip&sid=$schedid&plugin=$plugin&hostname=$hostname&rid=$res' target='_blank'>Add Exception</a>";
}

function printException($ip, $plugin, $dbconn, $excel = FALSE, $hostname = "",
                        $port = "") {
   //$esubmit = ( strpos($_SESSION['roles'],"esubmit") ) === false ? FALSE : TRUE;
   $esubmit = $_SESSION['roles']['esubmit'];
   if ($ip == "") {
      return "No IP specified to locate exceptions.";
   }
   // modified to look for exceptions that include the specific IP as
   //  well as any that are tagged for "ALL" hosts, not "Rejected" 
   //  exceptions, though.
   $equery="select eid, status, ip, submitDate
            from exceptions
            where ( ip LIKE '%$ip%' OR ip = 'ALL' )
               and plugin = '$plugin'
               and status <> 'Rejected'";
   if (is_numeric($port)) {
      $equery .= " and ( port = " . intval($port) . " OR port IS NULL )"; 
   }
   $eresult=$dbconn->execute($equery);
   if(!$eresult) { 
      return $dbconn->ErrorMsg(); 
   } else {
      $resArray = $eresult->GetArray();
      if (count($resArray) > 0) {
         foreach($resArray as $num=>$resData) {
            if($excel) {
               $foo .= "$resData[eid] - $resData[submitDate] - $resData[status]\n";
            } else {
               $foo .= "<a href='editException.php?op=view&eid=$resData[eid]' title='View Exception' onClick=\"popup('editException.php?op=view&eid=$resData[eid]','ViewException');return false;\">";
               $foo .= "$resData[eid]</a> - $resData[submitDate] - $resData[status]<br>\n";
            }
         }
      } else {
         $foo = "No Exceptions on file.";
         if($esubmit && !$excel) {
            $foo .= "<br><a href='editException.php?op=new&ip=$ip&sid=$schedid&plugin=$plugin&hostname=$hostname&port=$port' target='_blank'>Add Exception</a>";
         }
      }
      return $foo;
   }
}

function arrPortResultsCompare ($arrA, $arrB)
{
   if ($arrA[1]==$arrB[1]) { return 0; }
   return $arrA[1]>$arrB[1] ? 1 : -1;
}

function arrScanResultsCompare ($arrA, $arrB)
{
   global $sortby;

   if ($sortby==1) {
# Sort by port (tcp first, then udp) followed by risk (highest first)
      if (($arrA[1]==$arrB[1]) && ($arrA[2]==$arrB[2])) {
         if ($arrA[4]==$arrB[4]) { return 0; }
         if ($arrA[4]<$arrB[4]) { return -1; }
         if ($arrA[4]>$arrB[4]) { return 1; }
      }
      if (($arrA[1]==$arrB[1]) && ($arrA[2]>$arrB[2])) { return 1; }
      if (($arrA[1]==$arrB[1]) && ($arrA[2]<$arrB[2])) { return -1; }
      return $arrA[1]>$arrB[1] ? 1 : -1;
   } else {
# Sort by risk (highest first) followed by port (tcp first, then udp)
      if ($arrA[4]==$arrB[4]) {
         if (($arrA[1]==$arrB[1]) && ($arrA[2]==$arrB[2])) { return 0; }
         if (($arrA[1]==$arrB[1]) && ($arrA[2]>$arrB[2])) { return 1; }
         if (($arrA[1]==$arrB[1]) && ($arrA[2]<$arrB[2])) { return -1; }
         if ($arrA[1]>$arrB[1]) { return 1; }
         if ($arrA[1]<$arrB[1]) { return -1; }
      }
      return $arrA[4]>$arrB[4] ? 1 : -1;
   }
}

function make_link ($type,$data) {
   switch ($type) {
   case 'exception':
      return "<a href='editException.php?op=view&eid=$data'>$data</a>";
      break;
   default:
      break;
   }
}

// draws a form for data input
/*
 * requires:
 *   - array of form fields ( name => data)
 *      - data can be a string or an array
 *         - supported field types:  input, textarea, select, timeinput
 *   - [optional] array of hidden form fields
 *   - array of form buttons
 *
 * Example:
 *     $formFields = array( "Scan Request ID" => $requestID,
 *                          "Server Name" => array( "name" => "serverHostname",
 *                                                  "type" => "input",
 *                                                  "value" => $result[serverHostname]),
 *                          "Server IP" => array( "name" => "serverIP",
 *                                                  "type" => "input",
 *                                                  "value" => $result[serverIP]),
 *                          "Server O/S" => array( "name" => "serverOS",
 *                                                  "type" => "input",
 *                                                  "value" => $result[serverOS]),
 *                          "Server Location" => array( "name" => "serverLocation",
 *                                                  "type" => "select",
 *                                                  "value" => $serverLocationArray,
 *                                                  "selected" => $result[serverLocation]),
 *                          "Server Network" => array( "name" => "serverNetwork",
 *                                                  "type" => "select",
 *                                                  "value" => $serverNetworkArray,
 *                                                  "selected" => $result[serverNetwork]),
 *                          "Server State" => array( "name" => "serverState",
 *                                                  "type" => "select",
 *                                                  "value" => $serverStateArray,
 *                                                  "selected" => $result[serverState]),
 *                          "Desired Date" => array( "name" => "desiredScanDate",
 *                                                  "type" => "timeinput",
 *                                                  "value" => $result[desiredScanDate],
 *                                                  "comment" => "24-hour format") );
 *     $formButtons = array( "Submit" => array( "name" => "submit",
 *                                              "type" => "submit",
 *                                              "value" => "Update Scan Request"));
 *
 *     $formHidden = array( "requestID" => $requestID );
 *     $formData = array( "summary" => "Scan Request",
 *                        "fields" => $formFields,
 *                        "hidden" => $formHidden,
 *                        "buttons" => $formButtons);
 *
 *     drawForm($formData,$_SERVER['PHP_SELF']);
 */
function drawForm($data, $action, $method = "POST") {
   if (!empty($data)) {
      echo "<form action='$action' method='$method'>\n";
      echo "<table summary='$data[summary]'>\n";
      foreach ($data[fields] as $name=>$fd) {
         echo "<tr><th>$name</th>\n";
         if(is_array($fd)) {
            switch($fd['type']) {
            case 'input':
               if($fd[size] <> "") { $size = " size=$fd[size]"; }
               if($fd[maxsize] <> "") { $mazsize = " maxsize=$fd[maxsize]"; }
               echo "<td><input name='$fd[name]' $size $maxsize value='$fd[value]'></input>\n";
               break;
            case 'select':
               // for a select, the "value" element of the array is an
               // array itself
               if($fd[multiple] <> "") {
                  echo "<td><select name='$fd[name]' multiple size=$fd[multiple]>\n";
               } else {
                  echo "<td><select name='$fd[name]'>\n";
               }
               foreach ($fd[value] as $name=>$val) {
                  // if this is a one dimensional array, make the name = val
                  if($fd[nameIndex] == 0) { $name = $val; }
                  // if we have a "selected" element in the array
                  if(is_array($fd[selected])) {
                     if(in_array($name,$fd[selected])) {
                        echo "<option value='$name' selected>$val</option>\n";
                     } else {
                        echo "<option value='$name'>$val</option>\n";
                     }
                  } elseif($name == $fd[selected]) {
                     echo "<option value='$name' selected>$val</option>\n";
                  } else {
                     echo "<option value='$name'>$val</option>\n";
                  }
               }
               echo "</select>\n";
               break;
            case 'textarea':
               if($fd[cols] <> "") { $cols = " cols=$fd[cols]"; }
               if($fd[rows] <> "") { $rows = " rows=$fd[rows]"; }
               echo "<td><textarea name='$fd[name]' $cols $rows>$fd[value]</textarea>\n";
               break;
            case 'timeinput':
               // timeinput is a special input class
               // we want to print out select boxes for the month, day and year
               // and have a text box for the hour:min
               //  see if we have been given a starting value and set the
               //  initial values accordingly
               if(isset($fd[value])) {
                  list($dmy,$hms) = explode(" ",$fd[value]);
                  list($year,$mon,$day) = explode("-",$dmy);
                  list($hr,$min,$sec) = explode(":",$hms);
               }
               echo "<td>";
               echo "<select name='year'>\n";
               for($y=2007;$y<=2010;$y++){
                  if($y == $year) {
                     echo "<option value='$y' selected>$y</option>\n";
                  } else {
                     echo "<option value='$y'>$y</option>\n";
                  }
               }
               echo "</select>\n";
               echo "<select name='month'>\n";
               for($m=1;$m<=12;$m++) {
                  if($m == $mon) {
                     echo "<option value='$m' selected>$m</option>\n";
                  } else {
                     echo "<option value='$m'>$m</option>\n";
                  }
               }
               echo "</select>\n";
               echo "<select name='day'>\n";
               for($d=1;$d<=31;$d++) {
                  if($d == $day) {
                     echo "<option value='$d' selected>$d</option>\n";
                  } else {
                     echo "<option value='$d'>$d</option>\n";
                  }
               }
               echo "</select>\n";
               $time = ($time == "") ? "18:30" : "$hr:$min";
               echo "<input name='time' value='$time'></input>\n";
            }
            if(!empty($fd[comment])) {
               echo "&nbsp;&nbsp;$fd[comment]";
            }
            echo "</td>\n";
         } else {
            echo "<td>$fd</td>\n"; 
         }
         echo "</tr>\n";
      }
      echo "</table>\n";
      // now draw the form buttons
      foreach($data[buttons] as $name=>$fb) {
         switch ($fb[type]) {
            case 'submit':
               echo "<input type='submit' name='$fb[name]' value='$fb[value]'></input>\n";
               break;
         }
      }
      // last, tack on any hidden fields
      foreach($data[hidden] as $name=>$val) {
         echo "<input type='hidden' name='$name' value='$val'></input>\n";
      }
      echo "</form>\n";
   }
}

// draws a table for displaying data
/*
 * required:  mapping of field names to array elements
 *               [optional] array with links to additional functionality for
 *                 each array element
 *            array with data to be displayed
 *            table summary name
 *
 *  Example:
 *     $URL = "foo.php?sortby=%var%";
 *     $tableFieldMap = array( "Request ID" => "requestID",
 *                           "Hostname" => "serverHostname",
 *                           "IP" => "serverIP",
 *                           "Requestor" => array( 'var' => "requestorID",
                                                   'link' => 1),
 *                           "Request Date" => array( 'var' => "submitDate",
 *                                                    'link' => $URL),
 *                           "Desired Date" => "desiredScanDate",
 *                           "Change Control" => "changeControl",
 *                           "Status" => "requestStatus",
 *                           "Links" => array( "View" => array(
 *                                                   "url" => "scanRequest.php?op=view&requestID=",
 *                                                   "param" => "requestID",
 *                                                   "icon" => "images/zoom.png"),
 *                                             "Edit" => array(
 *                                                   "url" => "scanRequest.php?op=edit&requestID=",
 *                                                   "param" => "requestID",
 *                                                   "icon" => "images/pencil.png"),
 *                                             "Delete" => array(
 *                                                   "url" => "scanRequest.php?op=delete&requestID=",
 *                                                   "param" => "requestID",
 *                                                   "icon" => "images/delete.png")));
 *     drawTable($tableFieldMap, $results, "Scan Requests");
 */
function drawTable($map, $data, $summary = "") {
   echo "<table summary='$summary'>";
   echo "<tr>";
   foreach ($map as $fieldName=>$fieldValue) {
      if($fieldName != "Links") {
         // see if we need to create sorting links in the table headers
         if(is_array($fieldValue)) {
            $link = str_replace("%var%", $fieldValue['var'], $fieldValue['link']);
            echo "<th><a href='$link' text='Sort by $fieldName'>$fieldName</a></th>";
         } else {
            echo "<th>$fieldName</th>";
         }
      } else {
         echo "<th></th>";
      }
   }
   echo "</tr>";
   foreach($data as $entry) {
      echo "<tr>";
      foreach($map as $fieldName=>$fieldValue) {
         if($fieldName != "Links") {
            // some extra processing to allow custom links
            if(is_array($fieldValue)) {
               echo "<td>" . $entry[$fieldValue['var']] . "</td>";
            } else {
               echo "<td>$entry[$fieldValue]</td>";
            }
         } else {
            echo "<td>";
            foreach($fieldValue as $linkName=>$linkData) {
               $link = str_replace("%param%", $entry[$linkData['param']], $linkData['url']);
               echo "<a href='" . $link . 
                    "' alt='" . $linkName . "' title='" . $linkName . 
                    "'><img src='" .  $linkData['icon'] . "' alt='" . 
                    $linkName . "'></img></a>&nbsp;";
            }
            echo "</td>";
         }
      }
      echo "</tr>";
   }
   echo "</table>";
}
?>
